@@ -39,15 +39,17 @@ file_data_new (const char       *path,
 {
 	FileData *fd;
 
-	fd = g_new (FileData, 1);
+	fd = g_new0 (FileData, 1);
 
 	fd->ref = 1;
 	fd->path = g_strdup (path);
 	fd->name = file_name_from_path (fd->path);
 	fd->utf8_name = g_filename_to_utf8 (fd->name, -1, 0, 0, 0);
-	fd->size = info->size;
-	fd->ctime = info->ctime;
-	fd->mtime = info->mtime;
+	if (info != NULL) {
+		fd->size = info->size;
+		fd->ctime = info->ctime;
+		fd->mtime = info->mtime;
+	}
 	fd->error = FALSE;
 	fd->thumb = FALSE;
 	fd->comment = g_strdup ("");
@@ -65,7 +67,7 @@ file_data_update (FileData *fd)
 
 	g_return_if_fail (fd != NULL);
 
-	escaped = gnome_vfs_escape_path_string (fd->path);
+	escaped = escape_uri (fd->path);
 	info = gnome_vfs_file_info_new ();
 	result = gnome_vfs_get_file_info (escaped,
 					  info,
@@ -115,7 +117,7 @@ file_data_update_comment (FileData *fd)
 	if (fd->comment != NULL)
 		g_free (fd->comment);
 
-	data = comments_load_comment (fd->path);
+	data = comments_load_comment (fd->path, FALSE);
 
 	if (data == NULL) {
 		fd->comment = g_strdup ("");
@@ -154,6 +156,24 @@ file_data_unref (FileData *fd)
 }
 
 
+GList*
+file_data_list_dup (GList *list)
+{
+	GList *new_list = NULL, *scan;
+
+	if (list == NULL)
+		return NULL;
+
+	for (scan = list; scan; scan = scan->next) {
+		FileData *data = scan->data;
+		file_data_ref (data);
+		new_list = g_list_prepend (new_list, data);
+	}
+
+	return g_list_reverse (new_list);
+}
+
+
 void
 file_data_list_free (GList *list)
 {
