@@ -24,6 +24,7 @@
 #include <stdio.h>
 #include <glib.h>
 #include <glib/gprintf.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 #include "glib-utils.h"
 
 
@@ -372,3 +373,97 @@ debug (const char *file,
 #else /* ! DEBUG */
 #endif
 }
+
+
+GList *
+get_file_list_from_url_list (char *url_list)
+{
+	GList *list = NULL;
+	int    i;
+	char  *url_start, *url_end;
+
+	url_start = url_list;
+	while (url_start[0] != '\0') {
+		char *escaped;
+		char *unescaped;
+
+		if (strncmp (url_start, "file:", 5) == 0) {
+			url_start += 5;
+			if ((url_start[0] == '/') 
+			    && (url_start[1] == '/')) url_start += 2;
+		}
+
+		i = 0;
+		while ((url_start[i] != '\0')
+		       && (url_start[i] != '\r')
+		       && (url_start[i] != '\n')) i++;
+		url_end = url_start + i;
+
+		escaped = g_strndup (url_start, url_end - url_start);
+		unescaped = gnome_vfs_unescape_string_for_display (escaped);
+		g_free (escaped);
+
+		list = g_list_prepend (list, unescaped);
+
+		url_start = url_end;
+		i = 0;
+		while ((url_start[i] != '\0')
+		       && ((url_start[i] == '\r')
+			   || (url_start[i] == '\n'))) i++;
+		url_start += i;
+	}
+	
+	return g_list_reverse (list);
+}
+
+
+int
+strcmp_null_tollerant (const char *s1, const char *s2)
+{
+	if ((s1 == NULL) && (s2 == NULL))
+		return 0;
+	else if ((s1 != NULL) && (s2 == NULL))
+		return 1;
+	else if ((s1 == NULL) && (s2 != NULL))
+		return -1;
+	else 
+		return strcmp (s1, s2);
+}
+
+
+GList *
+_g_list_insert_list_before (GList *list1,
+			    GList *sibling,
+			    GList *list2)
+{
+  if (!list2)
+    {
+      return list1;
+    }
+  else if (!list1) 
+    {
+      return list2;
+    }
+  else if (sibling) 
+    {
+      GList *list2_last = g_list_last (list2);
+      if (sibling->prev)
+	{
+	  sibling->prev->next = list2;
+	  list2->prev = sibling->prev;
+	  sibling->prev = list2_last;
+	  list2_last->next = sibling;
+	  return list1;
+	}
+      else
+	{
+	  sibling->prev = list2_last;
+	  list2_last->next = sibling;
+	  return list2;
+	}
+    }
+  else
+    {
+      return g_list_concat (list1, list2);
+    }
+}
