@@ -28,13 +28,16 @@
 #include <time.h>
 #include <locale.h>
 #include <ctype.h>
+
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <libgnome/libgnome.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomevfs/gnome-vfs-directory.h>
+
 #include "catalog-web-exporter.h"
 #include "comments.h"
 #include "file-utils.h"
+#include "jpegutils/jpeg-data.h"
 #include "gth-exif-utils.h"
 #include "dlg-file-utils.h"
 #include "gthumb-init.h"
@@ -119,7 +122,7 @@ image_data_new (const char *filename)
 
 	idata = g_new (ImageData, 1);
 
-	cdata = comments_load_comment (filename);
+	cdata = comments_load_comment (filename, TRUE);
 	idata->comment = comments_get_comment_as_xml_string (cdata, "&nbsp;<br />", "&nbsp;<br />");
 	if (cdata != NULL)
 		comment_data_free (cdata);
@@ -351,8 +354,8 @@ catalog_web_exporter_get_type ()
 
 
 CatalogWebExporter *
-catalog_web_exporter_new (GThumbWindow *window,
-			  GList        *file_list)
+catalog_web_exporter_new (GthWindow *window,
+			  GList     *file_list)
 {
 	CatalogWebExporter *ce;
 	GList              *scan;
@@ -601,7 +604,7 @@ comp_func_size (gconstpointer a, gconstpointer b)
 static int
 comp_func_none (gconstpointer a, gconstpointer b)
 {
-	return 0;
+	return -1;
 }
 
 
@@ -1504,7 +1507,7 @@ export__copy_to_destination__step2 (GnomeVFSResult  result,
 	debug (DEBUG_INFO, "result: %s", gnome_vfs_result_to_string (result));
 
 	if (result != GNOME_VFS_OK) 
-		_gtk_error_dialog_run (GTK_WINDOW (ce->window->app), 
+		_gtk_error_dialog_run (GTK_WINDOW (ce->window), 
 				       gnome_vfs_result_to_string (result));
 
 	dlg_folder_delete (ce->window,
@@ -1607,6 +1610,39 @@ export__save_other_files (CatalogWebExporter *ce)
 }
 
 
+static void
+copy_exif_from_orig (const char *src_filename, 
+		     const char *dest_filename)
+{
+#if HAVE_LIBEXIF
+	JPEGData     *jdata_src, *jdata_dest;
+	ExifData     *edata_src;
+
+	jdata_src = jpeg_data_new_from_file (src_filename);
+	if (jdata_src == NULL)
+		return;
+
+	edata_src = jpeg_data_get_exif_data (jdata_src);
+	if (edata_src == NULL) {
+		jpeg_data_unref (jdata_src);
+		return;
+	}
+
+	jdata_dest = jpeg_data_new_from_file (dest_filename);
+	if (jdata_dest == NULL)
+		return;
+
+	jpeg_data_set_exif_data (jdata_dest, edata_src);
+
+	jpeg_data_save_file (jdata_dest, dest_filename);
+
+	exif_data_unref (edata_src);
+	jpeg_data_unref (jdata_src);
+	jpeg_data_unref (jdata_dest);
+#endif
+}
+
+
 static gboolean
 save_thumbnail_cb (gpointer data)
 {
@@ -1638,9 +1674,10 @@ save_thumbnail_cb (gpointer data)
 			if (_gdk_pixbuf_save (idata->thumb,
 					      filename,
 					      "jpeg",
-					      NULL, NULL))
+					      NULL, NULL)) {
+				copy_exif_from_orig (idata->src_filename, filename);
 				ce->album_files = g_list_prepend (ce->album_files, filename);
-			else
+			} else
 				g_free (filename);
 			
 			g_object_unref (idata->thumb);
@@ -1864,7 +1901,9 @@ load_next_file (CatalogWebExporter *ce)
 	ce->file_to_load = ce->file_to_load->next;
 	if (ce->file_to_load == NULL) {
 		/* sort list */
-		ce->file_list = g_list_sort (ce->file_list, get_sortfunc (ce));
+		if ((ce->sort_method != GTH_SORT_METHOD_NONE)
+		    && (ce->sort_method != GTH_SORT_METHOD_MANUAL))
+			ce->file_list = g_list_sort (ce->file_list, get_sortfunc (ce));
 		if (ce->sort_type == GTK_SORT_DESCENDING)
 			ce->file_list = g_list_reverse (ce->file_list);
 
@@ -1899,9 +1938,10 @@ save_image_preview_cb (gpointer data)
 			if (_gdk_pixbuf_save (idata->preview,
 					      filename,
 					      "jpeg",
-					      NULL, NULL))
+					      NULL, NULL)) {
+				copy_exif_from_orig (idata->src_filename, filename);
 				ce->album_files = g_list_prepend (ce->album_files, filename);
-			else
+			} else
 				g_free (filename);
 		}
 	}
@@ -1936,6 +1976,7 @@ save_resized_image_cb (gpointer data)
 					      filename,
 					      "jpeg",
 					      NULL, NULL)) {
+				copy_exif_from_orig (idata->src_filename, filename);
 				ce->album_files = g_list_prepend (ce->album_files, filename);
 				idata->file_size = get_file_size (filename);
 			} else
@@ -2326,31 +2367,6 @@ parse_theme_files (CatalogWebExporter *ce)
 }
 
 
-static char *
-get_temp_dir_name (void)
-{
-	static int  count = 0;
-	char       *tmp_dir = NULL;
-	
-	do {
-		g_free (tmp_dir);
-		tmp_dir = g_strdup_printf ("%s%s.%d.%d",
-					   g_get_tmp_dir (),
-					   "/gthumb",
-					   getpid (),
-					   count++);
-
-	} while (path_is_dir (tmp_dir));
-
-	if (mkdir (tmp_dir, 0700) != 0) {
-		g_free (tmp_dir);
-		return NULL;
-	}
-
-	return tmp_dir;
-}
-
-
 void
 catalog_web_exporter_export (CatalogWebExporter *ce)
 {
@@ -2366,7 +2382,7 @@ catalog_web_exporter_export (CatalogWebExporter *ce)
 	ce->tmp_location = get_temp_dir_name ();
 
 	if (ce->tmp_location == NULL) {
-		_gtk_error_dialog_run (GTK_WINDOW (ce->window->app), _("Could not create a temporary folder"));
+		_gtk_error_dialog_run (GTK_WINDOW (ce->window), _("Could not create a temporary folder"));
 		g_signal_emit (G_OBJECT (ce), catalog_web_exporter_signals[WEB_EXPORTER_DONE], 0);
 		return;
 	}
