@@ -3,7 +3,7 @@
 /*
  *  GThumb
  *
- *  Copyright (C) 2001 The Free Software Foundation, Inc.
+ *  Copyright (C) 2001, 2003 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,35 +22,79 @@
 
 #include <config.h>
 #include <string.h>
-#include <libgnome/libgnome.h>
+
+#include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs-types.h>
 #include <libgnomevfs/gnome-vfs-async-ops.h>
 
 #include "typedefs.h"
 #include "dir-list.h"
-#include "file-list.h"
+#include "gth-file-list.h"
 #include "file-data.h"
 #include "file-utils.h"
 #include "gconf-utils.h"
 #include "main.h"
 #include "pixbuf-utils.h"
 #include "icons/pixbufs.h"
+#include "gthumb-stock.h"
+#include "comments.h"
+
+
+#define DEF_SHOW_HIDDEN FALSE
 
 
 enum {
 	DIR_LIST_COLUMN_ICON,
 	DIR_LIST_COLUMN_NAME,
+	DIR_LIST_COLUMN_UTF_NAME,
 	DIR_LIST_NUM_COLUMNS
 };
 
 
+static void
+filename_cell_data_func (GtkTreeViewColumn *column,
+			 GtkCellRenderer   *renderer,
+			 GtkTreeModel      *model,
+			 GtkTreeIter       *iter,
+			 DirList           *dir_list)
+{
+	char *text;
+	GtkTreePath *path;
+	PangoUnderline underline;
+
+	gtk_tree_model_get (model, iter,
+			    DIR_LIST_COLUMN_UTF_NAME, &text,
+			    -1);
+
+	if (dir_list->single_click) {
+		path = gtk_tree_model_get_path (model, iter);
+
+		if (dir_list->hover_path == NULL ||
+		    gtk_tree_path_compare (path, dir_list->hover_path)) 
+			underline = PANGO_UNDERLINE_NONE;
+		else 
+			underline = PANGO_UNDERLINE_SINGLE;
+		
+		gtk_tree_path_free (path);
+
+	} else 
+		underline = PANGO_UNDERLINE_NONE;
+
+	g_object_set (G_OBJECT (renderer),
+		      "text", text,
+		      "underline", underline,
+		      NULL);
+
+	g_free (text);
+}
+
+
 static void
 add_columns (DirList     *dir_list,
 	     GtkTreeView *treeview)
 {
 	GtkCellRenderer   *renderer;
 	GtkTreeViewColumn *column;
-	GValue             value = { 0, };
 
 	/* The Name column. */
 
@@ -63,27 +107,105 @@ add_columns (DirList     *dir_list,
 					     NULL);
 	
 	dir_list->text_renderer = renderer = gtk_cell_renderer_text_new ();
-	
-	if (pref_get_real_click_policy () == CLICK_POLICY_SINGLE) {
-		g_value_init (&value, PANGO_TYPE_UNDERLINE);
-		g_value_set_enum (&value, PANGO_UNDERLINE_SINGLE);
-		g_object_set_property (G_OBJECT (renderer), "underline", &value);
-		g_value_unset (&value);
-	}
 
         gtk_tree_view_column_pack_start (column,
                                          renderer,
                                          TRUE);
         gtk_tree_view_column_set_attributes (column, renderer,
-                                             "text", DIR_LIST_COLUMN_NAME,
+                                             "text", DIR_LIST_COLUMN_UTF_NAME,
                                              NULL);
 
         gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);        
-	gtk_tree_view_column_set_sort_column_id (column, DIR_LIST_COLUMN_NAME);
+	gtk_tree_view_column_set_sort_column_id (column, DIR_LIST_COLUMN_UTF_NAME);
+	gtk_tree_view_column_set_cell_data_func (column, renderer,
+						 (GtkTreeCellDataFunc) filename_cell_data_func,
+						 dir_list, NULL);
+
         gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
 }
 
 
+static gboolean
+file_motion_notify_callback (GtkWidget *widget,
+			     GdkEventMotion *event,
+			     gpointer user_data)
+{
+	DirList     *dir_list = user_data;
+	GdkCursor   *cursor;
+	GtkTreePath *last_hover_path;
+	GtkTreeIter  iter;
+ 	
+	if (! dir_list->single_click) 
+		return FALSE;
+
+	if (event->window != gtk_tree_view_get_bin_window (GTK_TREE_VIEW (dir_list->list_view))) 
+                return FALSE;
+
+	last_hover_path = dir_list->hover_path;
+
+	gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (widget),
+				       event->x, event->y,
+				       &dir_list->hover_path,
+				       NULL, NULL, NULL);
+
+	if (dir_list->hover_path != NULL) 
+		cursor = gdk_cursor_new (GDK_HAND2);
+	else 
+		cursor = NULL;
+	
+	gdk_window_set_cursor (event->window, cursor);
+
+	/* only redraw if the hover row has changed */
+	if (!(last_hover_path == NULL && dir_list->hover_path == NULL) &&
+	    (!(last_hover_path != NULL && dir_list->hover_path != NULL) ||
+	     gtk_tree_path_compare (last_hover_path, dir_list->hover_path))) {
+		if (last_hover_path) {
+			gtk_tree_model_get_iter (GTK_TREE_MODEL (dir_list->list_store),
+						 &iter, last_hover_path);
+			gtk_tree_model_row_changed (GTK_TREE_MODEL (dir_list->list_store),
+						    last_hover_path, &iter);
+		}
+		
+		if (dir_list->hover_path) {
+			gtk_tree_model_get_iter (GTK_TREE_MODEL (dir_list->list_store),
+						 &iter, dir_list->hover_path);
+			gtk_tree_model_row_changed (GTK_TREE_MODEL (dir_list->list_store),
+						    dir_list->hover_path, &iter);
+		}
+	}
+	
+	gtk_tree_path_free (last_hover_path);
+
+ 	return FALSE;
+}
+
+
+static gboolean 
+file_leave_notify_callback (GtkWidget *widget,
+			    GdkEventCrossing *event,
+			    gpointer user_data)
+{
+	DirList     *dir_list = user_data;
+	GtkTreeIter  iter;
+
+	if (dir_list->single_click && (dir_list->hover_path != NULL)) {
+		gtk_tree_model_get_iter (GTK_TREE_MODEL (dir_list->list_store),
+					 &iter, 
+					 dir_list->hover_path);
+		gtk_tree_model_row_changed (GTK_TREE_MODEL (dir_list->list_store),
+					    dir_list->hover_path,
+					    &iter);
+
+		gtk_tree_path_free (dir_list->hover_path);
+		dir_list->hover_path = NULL;
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+
 DirList *
 dir_list_new ()
 {
@@ -91,7 +213,7 @@ dir_list_new ()
 	GtkTreeView *list_view;
 	GtkWidget   *scrolled;
 
-	dir_list = g_new (DirList, 1);
+	dir_list = g_new0 (DirList, 1);
 
 	/* Set default values. */
 
@@ -99,11 +221,14 @@ dir_list_new ()
 	dir_list->try_path = NULL;
 	dir_list->list = NULL;
 	dir_list->file_list = NULL;
-	dir_list->show_dot_files = eel_gconf_get_boolean (PREF_SHOW_HIDDEN_FILES);
+	dir_list->show_dot_files = eel_gconf_get_boolean (PREF_SHOW_HIDDEN_FILES, DEF_SHOW_HIDDEN);
 	dir_list->old_dir = NULL;
 	dir_list->dir_load_handle = NULL;
 	dir_list->result = GNOME_VFS_OK;
 
+	dir_list->single_click = (pref_get_real_click_policy () == GTH_CLICK_POLICY_SINGLE);
+	dir_list->hover_path = NULL;
+
 	/* Create the widgets. */
 
 	scrolled = gtk_scrolled_window_new (NULL, NULL);
@@ -115,13 +240,23 @@ dir_list_new ()
 
 	dir_list->list_store = gtk_list_store_new (DIR_LIST_NUM_COLUMNS,
 						   GDK_TYPE_PIXBUF,
+						   G_TYPE_STRING,
 						   G_TYPE_STRING);
 	list_view = (GtkTreeView*) gtk_tree_view_new_with_model (GTK_TREE_MODEL (dir_list->list_store));
         gtk_tree_view_set_rules_hint (list_view, FALSE);
         add_columns (dir_list, list_view);
 	gtk_tree_view_set_headers_visible (list_view, FALSE);
         gtk_tree_view_set_enable_search (list_view, TRUE);
-        gtk_tree_view_set_search_column (list_view, DIR_LIST_COLUMN_NAME);
+        gtk_tree_view_set_search_column (list_view, DIR_LIST_COLUMN_UTF_NAME);
+
+	g_signal_connect (G_OBJECT (list_view), 
+			  "motion_notify_event",
+			  G_CALLBACK (file_motion_notify_callback), 
+			  dir_list);
+	g_signal_connect (G_OBJECT (list_view), 
+			  "leave_notify_event",
+			  G_CALLBACK (file_leave_notify_callback), 
+			  dir_list);
 
 	/**/
 
@@ -136,20 +271,15 @@ dir_list_new ()
 void
 dir_list_update_underline (DirList *dir_list)
 {
-	GValue value = { 0, };
+	GdkWindow  *win = gtk_tree_view_get_bin_window (GTK_TREE_VIEW (dir_list->list_view));
+	GdkDisplay *display;
 
-	g_value_init (&value, PANGO_TYPE_UNDERLINE);
+	dir_list->single_click = (pref_get_real_click_policy () == GTH_CLICK_POLICY_SINGLE);
 
-	if (pref_get_real_click_policy () == CLICK_POLICY_SINGLE) 
-		g_value_set_enum (&value, PANGO_UNDERLINE_SINGLE);
-	else
-		g_value_set_enum (&value, PANGO_UNDERLINE_NONE);
-
-	g_object_set_property (G_OBJECT (dir_list->text_renderer), 
-			       "underline", &value);
-	g_value_unset (&value);
-
-	gtk_widget_queue_draw (dir_list->root_widget);
+	gdk_window_set_cursor (win, NULL);
+	display = gtk_widget_get_display (GTK_WIDGET (dir_list->list_view));
+	if (display != NULL) 
+		gdk_display_flush (display);
 }
 
 
@@ -184,17 +314,93 @@ dir_list_free (DirList *dir_list)
 }
 
 
+static gboolean
+is_a_film (DirList    *dir_list,
+	   const char *name)
+{
+	gboolean   film = FALSE;
+	char      *folder;
+
+	folder = g_build_filename (dir_list->path, name, NULL);
+	film = folder_is_film (folder);
+	g_free (folder);
+
+	return film;
+}
+
+
+static void
+dir_list_update_view (DirList *dir_list)
+{
+	GdkPixbuf *dir_pixbuf;
+	GdkPixbuf *up_pixbuf;
+	GdkPixbuf *film_pixbuf;
+	GList     *scan;
+
+	dir_pixbuf = get_folder_pixbuf (get_folder_pixbuf_size_for_list (dir_list->list_view));
+	up_pixbuf = gtk_widget_render_icon (dir_list->list_view,
+					    GTK_STOCK_GO_UP,
+					    GTK_ICON_SIZE_MENU,
+					    NULL);
+	film_pixbuf = gtk_widget_render_icon (dir_list->list_view,
+					      GTHUMB_STOCK_FILM,
+					      GTK_ICON_SIZE_MENU,
+					      NULL);
+
+	gtk_list_store_clear (dir_list->list_store);
+
+	for (scan = dir_list->list; scan; scan = scan->next) {
+		char        *name = scan->data;
+		char        *utf8_name;
+		GtkTreeIter  iter;
+		GdkPixbuf   *pixbuf;
+
+		if (strcmp (name, "..") == 0)
+			pixbuf = dir_pixbuf /*up_pixbuf*/;
+		else if (is_a_film (dir_list, name))
+			pixbuf = film_pixbuf;
+		else
+			pixbuf = dir_pixbuf;
+
+		utf8_name = g_filename_display_name (name);
+		gtk_list_store_append (dir_list->list_store, &iter);
+		gtk_list_store_set (dir_list->list_store, &iter,
+				    DIR_LIST_COLUMN_ICON, pixbuf,
+				    DIR_LIST_COLUMN_UTF_NAME, utf8_name,
+				    DIR_LIST_COLUMN_NAME, name,
+				    -1);
+
+		g_free (utf8_name);
+	}
+
+	g_object_unref (dir_pixbuf);
+	g_object_unref (film_pixbuf);
+	g_object_unref (up_pixbuf);
+}
+
+
+void
+dir_list_update_icon_theme (DirList *dir_list)
+{
+	dir_list_update_view (dir_list);
+}
+
+
 static void
 dir_list_refresh_continue (PathListData *pld, 
-			   gpointer data)
+			   gpointer      data)
 {
-	GList     *scan;
+	DirList   *dir_list = data;
 	GList     *new_dir_list = NULL;
 	GList     *new_file_list = NULL;
 	GList     *filtered;
-	DirList   *dir_list;
-	GdkPixbuf *dir_pixbuf;
-	GdkPixbuf *up_pixbuf;
+
+	if (pld == NULL) {
+		if (dir_list->done_func) 
+			dir_list->done_func (dir_list, dir_list->done_data);
+		dir_list->done_func = NULL;
+		return;
+	}
 
 	dir_list = data;
 	dir_list->result = pld->result;
@@ -264,7 +470,7 @@ dir_list_refresh_continue (PathListData *pld,
 	/* Set the new dir list */
 
 	filtered = dir_list_filter_and_sort (new_dir_list, TRUE, 
-					     eel_gconf_get_boolean (PREF_SHOW_HIDDEN_FILES));
+					     eel_gconf_get_boolean (PREF_SHOW_HIDDEN_FILES, DEF_SHOW_HIDDEN));
 
 	/* * Add the ".." entry if the current path is not "/". 
 	 * path_list_new does not include the "." and ".." elements. */
@@ -279,36 +485,7 @@ dir_list_refresh_continue (PathListData *pld,
 
 	/* Update the view. */
 
-	dir_pixbuf = get_folder_pixbuf (LIST_ICON_SIZE);
-	up_pixbuf = gtk_widget_render_icon (dir_list->list_view,
-					    GTK_STOCK_GO_UP,
-					    GTK_ICON_SIZE_MENU,
-					    NULL);
-	gtk_list_store_clear (dir_list->list_store);
-
-	for (scan = dir_list->list; scan; scan = scan->next) {
-		char        *name = scan->data;
-		char        *utf8_name;
-		GtkTreeIter  iter;
-		GdkPixbuf   *pixbuf;
-
-		if (strcmp (name, "..") == 0)
-			pixbuf = up_pixbuf;
-		else 
-			pixbuf = dir_pixbuf;
-
-		utf8_name = g_locale_to_utf8 (name, -1, NULL, NULL, NULL);
-		gtk_list_store_append (dir_list->list_store, &iter);
-		gtk_list_store_set (dir_list->list_store, &iter,
-				    DIR_LIST_COLUMN_ICON, pixbuf,
-				    DIR_LIST_COLUMN_NAME, utf8_name,
-				    -1);
-
-		g_free (utf8_name);
-	}
-
-	g_object_unref (dir_pixbuf);
-	g_object_unref (up_pixbuf);
+	dir_list_update_view (dir_list);
 
 	/* Make past dir visible in the list. */
 
@@ -336,6 +513,7 @@ dir_list_refresh_continue (PathListData *pld,
 						      TRUE,
 						      0.5,
 						      0.0);
+			gtk_tree_selection_select_path (gtk_tree_view_get_selection (GTK_TREE_VIEW (dir_list->list_view)), path);
 			gtk_tree_path_free (path);
 		}
 		g_free (dir_list->old_dir);
@@ -391,6 +569,8 @@ dir_list_add_directory (DirList         *dir_list,
 	const char  *name_only;
 	GList       *scan;
 	GdkPixbuf   *dir_pixbuf;
+	GdkPixbuf   *film_pixbuf;
+	GdkPixbuf   *pixbuf;
 	char        *utf8_name;
 	GtkTreeIter  iter;
 	int          pos;
@@ -409,7 +589,7 @@ dir_list_add_directory (DirList         *dir_list,
 
 	/* insert dir in the list */
 	
-	if (! (file_is_hidden (name_only) && ! eel_gconf_get_boolean (PREF_SHOW_HIDDEN_FILES))
+	if (! (file_is_hidden (name_only) && ! eel_gconf_get_boolean (PREF_SHOW_HIDDEN_FILES, DEF_SHOW_HIDDEN))
 	    && (strcmp (name_only, CACHE_DIR) != 0)) 
 		dir_list->list = g_list_prepend (dir_list->list, g_strdup (name_only));
 	dir_list->list = g_list_sort (dir_list->list, (GCompareFunc) strcasecmp);
@@ -425,15 +605,24 @@ dir_list_add_directory (DirList         *dir_list,
 
 	/* insert dir in the list view */
 
-	dir_pixbuf = get_folder_pixbuf (LIST_ICON_SIZE);
-	utf8_name = g_locale_to_utf8 (name_only, -1, NULL, NULL, NULL);
+	dir_pixbuf = get_folder_pixbuf (get_folder_pixbuf_size_for_list (dir_list->list_view));
+	film_pixbuf = gtk_widget_render_icon (dir_list->list_view, GTHUMB_STOCK_FILM, GTK_ICON_SIZE_MENU, NULL);
+
+	if (is_a_film (dir_list, name_only))
+		pixbuf = film_pixbuf;
+	else
+		pixbuf = dir_pixbuf;
+
+	utf8_name = g_filename_display_name (name_only);
 	gtk_list_store_insert (dir_list->list_store, &iter, pos);
 	gtk_list_store_set (dir_list->list_store, &iter,
-			    DIR_LIST_COLUMN_ICON, dir_pixbuf,
-			    DIR_LIST_COLUMN_NAME, utf8_name,
+			    DIR_LIST_COLUMN_ICON, pixbuf,
+			    DIR_LIST_COLUMN_UTF_NAME, utf8_name,
+			    DIR_LIST_COLUMN_NAME, name_only,
 			    -1);
 	g_free (utf8_name);
-	g_object_unref (dir_pixbuf);	
+	g_object_unref (dir_pixbuf);
+	g_object_unref (film_pixbuf);
 }
 
 
@@ -492,7 +681,7 @@ dir_list_get_name_from_iter (DirList     *dir_list,
 
 	gtk_tree_model_get (GTK_TREE_MODEL (dir_list->list_store), 
 			    iter,
-			    DIR_LIST_COLUMN_NAME, &utf8_name,
+			    DIR_LIST_COLUMN_UTF_NAME, &utf8_name,
 			    -1);
 
 	return utf8_name;
@@ -503,21 +692,17 @@ gchar *
 dir_list_get_path_from_iter (DirList     *dir_list,
 			     GtkTreeIter *iter)
 {
-	gchar *utf8_name;
-	gchar *name;
-	gchar *new_path;
+	char *name;
+	char *new_path;
 
 	gtk_tree_model_get (GTK_TREE_MODEL (dir_list->list_store), 
 			    iter,
-			    DIR_LIST_COLUMN_NAME, &utf8_name,
+			    DIR_LIST_COLUMN_NAME, &name,
 			    -1);
 
-	if (utf8_name == NULL) 
+	if (name == NULL) 
 		return NULL;
 
-	name = g_locale_from_utf8 (utf8_name, -1, NULL, NULL, NULL);
-	g_free (utf8_name);
-
 	if (strcmp (name, ".") == 0)
 		new_path = g_strdup (dir_list->path);
 	else if (strcmp (name, "..") == 0)
