@@ -3,7 +3,7 @@
 /*
  *  GThumb
  *
- *  Copyright (C) 2001, 2003 Free Software Foundation, Inc.
+ *  Copyright (C) 2001, 2003, 2004 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,22 +23,28 @@
 #include <config.h>
 #include <string.h>
 
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#include <libgnome/libgnome.h>
-#include <libgnomeui/gnome-dialog.h>
-#include <libgnomeui/gnome-dialog-util.h>
-#include <libgnomeui/gnome-dateedit.h>
+#include <libgnomeui/gnome-help.h>
 #include <glade/glade.h>
+
 #include "typedefs.h"
+#include "file-utils.h"
+#include "preferences.h"
 #include "main.h"
-#include "gthumb-window.h"
+#include "gth-window.h"
 #include "gtk-utils.h"
 #include "gtkcellrendererthreestates.h"
 #include "gth-file-view.h"
 #include "comments.h"
+#include "dlg-categories.h"
 
 
+typedef void (*SaveFunc) (GList *file_list, gpointer data);
+
+
+#define DIALOG_NAME "categories"
 #define GLADE_FILE "gthumb_comments.glade"
 #define CATEGORY_SEPARATOR ";"
 
@@ -53,12 +59,20 @@ enum {
 
 
 typedef struct {
-	GThumbWindow  *window;
 	GladeXML      *gui;
 	GList         *file_list;
+	GList         *default_categories_list;
+	GList        **add_list;
+	GList        **remove_list;
+	SaveFunc       save_func;
+	gpointer       save_data;
+	DoneFunc       done_func;
+	gpointer       done_data;
 
+	GthWindow     *window;
 	GtkWidget     *dialog;
 
+	GtkWidget     *c_ok_button;
 	GtkWidget     *keyword_entry;
 	GtkWidget     *add_key_button;
 	GtkWidget     *remove_key_button;
@@ -69,22 +83,51 @@ typedef struct {
 } DialogData;
 
 
+
+static void
+free_dialog_data (DialogData *data)
+{
+	if (data->file_list != NULL) {
+		g_list_foreach (data->file_list, (GFunc) g_free, NULL);
+		g_list_free (data->file_list);
+		data->file_list = NULL;
+	}
+	
+	if (data->original_cdata != NULL) {
+		comment_data_free (data->original_cdata);
+		data->original_cdata = NULL;
+	}
+}
+
+
 /* called when the main dialog is closed. */
 static void
 destroy_cb (GtkWidget  *widget, 
 	    DialogData *data)
 {
-	data->window->categories_dlg = NULL;
+	if (data->window != NULL)
+		gth_window_set_categories_dlg (data->window, NULL);
 
 	g_object_unref (data->gui);
-	g_list_foreach (data->file_list, (GFunc) g_free, NULL);
-	g_list_free (data->file_list);
-	comment_data_free (data->original_cdata);
+	free_dialog_data (data);
+	path_list_free (data->default_categories_list);
+
+	if (data->done_func)
+		(*data->done_func) (data->done_data);
 
 	g_free (data);
 }
 
 
+static gboolean
+unrealize_cb (GtkWidget  *widget, 
+	  DialogData *data)
+{
+	pref_util_save_window_geometry (GTK_WINDOW (widget), DIALOG_NAME);
+	return FALSE;
+}
+
+
 static void
 update_category_entry (DialogData *data)
 {
@@ -216,10 +259,11 @@ add_category_cb (GtkWidget  *widget,
 		gtk_list_store_append (data->keywords_list_model, &iter);
 		gtk_list_store_set (data->keywords_list_model, &iter,
 				    IS_EDITABLE_COLUMN, TRUE,
-				    USE_CATEGORY_COLUMN, 0,
+				    USE_CATEGORY_COLUMN, 1,
 				    CATEGORY_COLUMN, new_category,
 				    -1);
 		update_category_entry (data);
+		gtk_widget_set_sensitive (data->c_ok_button, TRUE);
 	}
 
 	g_free (new_category);
@@ -240,6 +284,7 @@ remove_category_cb (GtkWidget *widget,
 
 	gtk_list_store_remove (data->keywords_list_model, &iter);
 	update_category_entry (data);
+	gtk_widget_set_sensitive (data->c_ok_button, TRUE);
 }
 
 
@@ -252,6 +297,7 @@ save_categories (DialogData *data)
 
 	cat_list = get_categories (data->keywords_list_model, -1);
 	categories = bookmarks_new (RC_CATEGORIES_FILE);
+	bookmarks_set_max_lines (categories, -1);
 
 	for (scan = cat_list; scan; scan = scan->next) {
 		char *category = scan->data;
@@ -280,40 +326,36 @@ static void
 ok_clicked_cb (GtkWidget  *widget, 
 	       DialogData *data)
 {
-	GList *add_list, *remove_list, *scan;
-
 	save_categories (data);
 
-	add_list = get_categories (data->keywords_list_model, 1);
-	remove_list = get_categories (data->keywords_list_model, 0);
-
-	for (scan = data->file_list; scan; scan = scan->next) {
-		const char  *filename = scan->data;
-		CommentData *cdata;
-		GList       *scan2;
-
-		cdata = comments_load_comment (filename);
-		if (cdata == NULL)
-			cdata = comment_data_new ();
-		else 
-			for (scan2 = remove_list; scan2; scan2 = scan2->next) {
-				const char *k = scan2->data;
-				comment_data_remove_keyword (cdata, k);
-			}
-
-		for (scan2 = add_list; scan2; scan2 = scan2->next) {
-			const char *k = scan2->data;
-			comment_data_add_keyword (cdata, k);
-		}
-
-		comments_save_categories (filename, cdata);
-		comment_data_free (cdata);
+	if (data->add_list != NULL) {
+		if (*(data->add_list) != NULL)
+			path_list_free (*(data->add_list));
+		*(data->add_list) = get_categories (data->keywords_list_model, 1);
 	}
 
-	path_list_free (add_list);
-	path_list_free (remove_list);
+	if (data->remove_list != NULL) {
+		if (*(data->remove_list) != NULL)
+			path_list_free (*(data->remove_list));
+		*(data->remove_list) = get_categories (data->keywords_list_model, 0);
+	}
 
-	gtk_widget_destroy (data->dialog);
+	if (data->save_func) {
+		(*data->save_func) (data->file_list, data->save_data);
+		gtk_widget_set_sensitive (data->c_ok_button, FALSE);
+		if (data->window != NULL) {
+			gboolean close_on_save;
+
+			gth_window_update_current_image_metadata (data->window);
+
+			g_object_get (data->window,
+				      "close_on_save", &close_on_save,
+				      NULL);
+			if (close_on_save)
+				dlg_categories_close (data->dialog);
+		}
+	} else
+		gtk_widget_destroy (data->dialog);
 }
 
 
@@ -367,6 +409,7 @@ use_category_toggled (GtkCellRendererThreeStates *cell,
 	
 	gtk_tree_path_free (path);
 	update_category_entry (data);
+	gtk_widget_set_sensitive (data->c_ok_button, TRUE);
 }
 
 
@@ -444,60 +487,89 @@ name_column_sort_func (GtkTreeModel *model,
 }
 
 
+static gboolean
+keyword_equal_func (GtkTreeModel *model,
+		    gint          column, 
+		    const gchar  *key,
+		    GtkTreeIter  *iter,
+		    gpointer      search_data)
+{
+	char *cell;
+	gtk_tree_model_get (model, iter, column, &cell, -1);
+	return g_strcasecmp (key, cell) > 0;
+}
+    
+
 
-void
-dlg_categories (GtkWidget *widget, 
-		gpointer   wdata)
+
+
+static GtkWidget*
+dlg_categories_common (GtkWindow     *parent,
+		       GthWindow     *window,
+		       GList         *file_list,
+		       GList         *default_categories_list,
+		       GList        **add_categories_list,
+		       GList        **remove_categories_list,
+		       SaveFunc       save_func,
+		       gpointer       save_data,
+		       DoneFunc       done_func,
+		       gpointer       done_data,
+		       gboolean       modal)
 {
-	GThumbWindow      *window = wdata;
 	DialogData        *data;
 	GtkWidget         *btn_ok;
 	GtkWidget         *btn_cancel;
 	GtkWidget         *btn_help;
-	CommentData       *cdata;
-	GList             *scan;
 	GtkCellRenderer   *renderer;
 	GtkTreeViewColumn *column;
-	char              *first_image;
-	GList             *other_keys = NULL;
 
-	if (window->categories_dlg != NULL) {
-		gtk_window_present (GTK_WINDOW (window->categories_dlg));
-		return;
-	}
 
 	data = g_new (DialogData, 1);
 
 	data->window = window;
+	data->default_categories_list = path_list_dup (default_categories_list);
+	data->add_list = add_categories_list;
+	data->remove_list = remove_categories_list;
+	data->save_func = save_func;
+	data->save_data = save_data;
+	data->done_func = done_func;
+	data->done_data = done_data;
+	data->original_cdata = NULL;
 
 	data->gui = glade_xml_new (GTHUMB_GLADEDIR "/" GLADE_FILE , NULL, NULL);
         if (!data->gui) {
+		g_free (data);
                 g_warning ("Could not find " GLADE_FILE "\n");
-                return;
+                return NULL;
         }
 
-	data->file_list = gth_file_view_get_file_list_selection (window->file_list->view);
-	if (data->file_list == NULL) {
-		g_free (data);
-		return;
-	}
+	if (file_list != NULL)
+		data->file_list = path_list_dup (file_list);
+	else
+		data->file_list = NULL;
 
 	/* Get the widgets. */
 
 	data->dialog = glade_xml_get_widget (data->gui, "categories_dialog");
-	window->categories_dlg = data->dialog;
 
 	data->keyword_entry = glade_xml_get_widget (data->gui, "c_keyword_entry");
 	data->add_key_button = glade_xml_get_widget (data->gui, "c_add_key_button");
 	data->remove_key_button = glade_xml_get_widget (data->gui, "c_remove_key_button");
 	data->keywords_list_view = glade_xml_get_widget (data->gui, "c_keywords_treeview");
 
-	btn_ok = glade_xml_get_widget (data->gui, "c_ok_button");
+	data->c_ok_button = btn_ok = glade_xml_get_widget (data->gui, "c_ok_button");
 	btn_cancel = glade_xml_get_widget (data->gui, "c_cancel_button");
 	btn_help = glade_xml_get_widget (data->gui, "c_help_button");
 
+	if (!modal) {
+		gtk_button_set_label (GTK_BUTTON (btn_ok), GTK_STOCK_SAVE);
+		gtk_button_set_label (GTK_BUTTON (btn_cancel), GTK_STOCK_CLOSE);
+	}
+
 	/* Set widgets data. */
 
+	g_object_set_data (G_OBJECT (data->dialog), "dialog_data", data);
+
 	data->keywords_list_model = gtk_list_store_new (NUM_COLUMNS,
 							G_TYPE_BOOLEAN,
 							G_TYPE_BOOLEAN,
@@ -508,6 +580,13 @@ dlg_categories (GtkWidget *widget,
 	g_object_unref (data->keywords_list_model);
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (data->keywords_list_view), FALSE);
 
+	gtk_tree_view_set_search_column (GTK_TREE_VIEW (data->keywords_list_view), 
+					 CATEGORY_COLUMN);
+	gtk_tree_view_set_search_equal_func (GTK_TREE_VIEW (data->keywords_list_view),
+					     keyword_equal_func,
+					     NULL,
+					     NULL);
+
 	renderer = gtk_cell_renderer_three_states_new ();
 	g_signal_connect (G_OBJECT (renderer), 
 			  "toggled",
@@ -535,11 +614,204 @@ dlg_categories (GtkWidget *widget,
 
 	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (data->keywords_list_model), GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, GTK_SORT_ASCENDING);
 
+	/* Set the signals handlers. */
+	
+	g_signal_connect (G_OBJECT (data->dialog), 
+			  "destroy",
+			  G_CALLBACK (destroy_cb),
+			  data);
+	g_signal_connect (G_OBJECT (data->dialog), 
+			  "unrealize",
+			  G_CALLBACK (unrealize_cb),
+			  data);
+	g_signal_connect (G_OBJECT (btn_ok), 
+			  "clicked",
+			  G_CALLBACK (ok_clicked_cb),
+			  data);
+	g_signal_connect (G_OBJECT (btn_cancel), 
+			  "clicked",
+			  G_CALLBACK (cancel_clicked_cb),
+			  data);
+	g_signal_connect (G_OBJECT (btn_help), 
+			  "clicked",
+			  G_CALLBACK (help_cb),
+			  data);
+
+	g_signal_connect (G_OBJECT (data->add_key_button), 
+			  "clicked",
+			  G_CALLBACK (add_category_cb),
+			  data);
+	g_signal_connect (G_OBJECT (data->remove_key_button), 
+			  "clicked",
+			  G_CALLBACK (remove_category_cb),
+			  data);
+
+	/* run dialog. */
+
+	if (parent != NULL)
+		gtk_window_set_transient_for (GTK_WINDOW (data->dialog), parent);
+
+	gtk_window_set_modal (GTK_WINDOW (data->dialog), modal);
+	pref_util_restore_window_geometry (GTK_WINDOW (data->dialog), DIALOG_NAME);
+	dlg_categories_update (data->dialog);
+
+	gtk_widget_grab_focus (data->keywords_list_view);
+
+	return data->dialog;
+}
+
+
+/**/
+
+
+void
+dlg_choose_categories (GtkWindow     *parent,
+		       GList         *file_list,
+		       GList         *default_categories_list,
+		       GList        **add_categories_list,
+		       GList        **remove_categories_list,
+		       DoneFunc       done_func,
+		       gpointer       done_data)
+{
+	dlg_categories_common (parent,
+			       NULL,
+			       file_list,
+			       default_categories_list,
+			       add_categories_list,
+			       remove_categories_list,
+			       NULL,
+			       NULL,
+			       done_func,
+			       done_data,
+			       TRUE);
+}
+
+
+/**/
+
+
+typedef struct {
+	GthWindow     *window;
+	GList         *add_list;
+	GList         *remove_list;
+} DlgCategoriesData;
+
+
+static void
+dlg_categories__done (gpointer data)
+{
+	DlgCategoriesData *dcdata = data;
+
+	path_list_free (dcdata->add_list);
+	dcdata->add_list = NULL;
+	
+	path_list_free (dcdata->remove_list);
+	dcdata->remove_list = NULL;
+
+	g_free (dcdata);
+}
+
+
+static void
+dlg_categories__save (GList    *file_list,
+		      gpointer  data)
+{
+	DlgCategoriesData *dcdata = data;
+	GList             *scan;
+	
+	for (scan = file_list; scan; scan = scan->next) {
+		const char  *filename = scan->data;
+		CommentData *cdata;
+		GList       *scan2;
+
+		cdata = comments_load_comment (filename, TRUE);
+		if (cdata == NULL)
+			cdata = comment_data_new ();
+		else 
+			for (scan2 = dcdata->remove_list; scan2; scan2 = scan2->next) {
+				const char *k = scan2->data;
+				comment_data_remove_keyword (cdata, k);
+			}
+
+		for (scan2 = dcdata->add_list; scan2; scan2 = scan2->next) {
+			const char *k = scan2->data;
+			comment_data_add_keyword (cdata, k);
+		}
+
+		comments_save_categories (filename, cdata);
+		comment_data_free (cdata);
+	}
+
+	path_list_free (dcdata->add_list);
+	dcdata->add_list = NULL;
+
+	path_list_free (dcdata->remove_list);
+	dcdata->remove_list = NULL;
+}
+
+
+GtkWidget*
+dlg_categories_new (GthWindow *window)
+{
+	GtkWidget         *current_dlg;
+	DlgCategoriesData *dcdata;
+
+	current_dlg = gth_window_get_categories_dlg (window);
+	if (current_dlg != NULL) {
+		gtk_window_present (GTK_WINDOW (current_dlg));
+		return current_dlg;
+	}
+
+	dcdata = g_new0 (DlgCategoriesData, 1);
+	dcdata->window = window;
+	dcdata->add_list = NULL;
+	dcdata->remove_list = NULL;
+
+	return dlg_categories_common (GTK_WINDOW (window),
+				      window,
+				      NULL,
+				      NULL,
+				      &(dcdata->add_list),
+				      &(dcdata->remove_list),
+				      dlg_categories__save,
+				      dcdata,
+				      dlg_categories__done,
+				      dcdata,
+				      FALSE);
+}
+
+
+void
+dlg_categories_update (GtkWidget *dlg)
+{
+	DialogData    *data;
+	CommentData   *cdata = NULL;
+	GList         *scan;
+	GList         *other_keys = NULL;
+
+
+	g_return_if_fail (dlg != NULL);
+
+	data = g_object_get_data (G_OBJECT (dlg), "dialog_data");
+	g_return_if_fail (data != NULL);
+
+	gtk_list_store_clear (data->keywords_list_model);
+
 	/**/
 
-	first_image = data->file_list->data;
-	data->original_cdata = cdata = comments_load_comment (first_image);
+	if (data->window != NULL) {
+		free_dialog_data (data);
+		data->file_list = gth_window_get_file_list_selection (data->window);
+	} else if (data->original_cdata != NULL) {
+		comment_data_free (data->original_cdata);
+		data->original_cdata = NULL;
+	}
 
+	if (data->file_list != NULL) {
+		char *first_image = data->file_list->data;
+		data->original_cdata = cdata = comments_load_comment (first_image, TRUE);
+	}
+	
 	if (cdata != NULL) {
 		comment_data_free_comment (cdata);
 
@@ -548,7 +820,7 @@ dlg_categories (GtkWidget *widget,
 			CommentData *scan_cdata;
 			int          i;
 
-			scan_cdata = comments_load_comment (scan->data);
+			scan_cdata = comments_load_comment (scan->data, TRUE);
 		
 			if (scan_cdata == NULL) {
 				comment_data_free_keywords (cdata);
@@ -580,7 +852,7 @@ dlg_categories (GtkWidget *widget,
 		CommentData *scan_cdata;
 		int          j;
 
-		scan_cdata = comments_load_comment (scan->data);
+		scan_cdata = comments_load_comment (scan->data, TRUE);
 		
 		if (scan_cdata == NULL) 
 			continue;
@@ -624,6 +896,21 @@ dlg_categories (GtkWidget *widget,
 		}
 	}
 
+	for (scan = data->default_categories_list; scan; scan = scan->next) {
+		char        *keyword = scan->data;
+		GtkTreeIter  iter;
+		
+		gtk_list_store_append (data->keywords_list_model, 
+				       &iter);
+		
+		gtk_list_store_set (data->keywords_list_model, &iter,
+				    IS_EDITABLE_COLUMN, FALSE,
+				    HAS_THIRD_STATE_COLUMN, FALSE,
+				    USE_CATEGORY_COLUMN, 1,
+				    CATEGORY_COLUMN, keyword,
+				    -1);
+	}
+
 	for (scan = other_keys; scan; scan = scan->next) {
 		char        *keyword = scan->data;
 		GtkTreeIter  iter;
@@ -644,47 +931,19 @@ dlg_categories (GtkWidget *widget,
 
 	add_saved_categories (data);
 	update_category_entry (data);
+	gtk_widget_set_sensitive (data->c_ok_button, FALSE);
+}
 
-	/* Set the signals handlers. */
-	
-	g_signal_connect (G_OBJECT (data->dialog), 
-			  "destroy",
-			  G_CALLBACK (destroy_cb),
-			  data);
-	g_signal_connect (G_OBJECT (btn_ok), 
-			  "clicked",
-			  G_CALLBACK (ok_clicked_cb),
-			  data);
-	g_signal_connect (G_OBJECT (btn_cancel), 
-			  "clicked",
-			  G_CALLBACK (cancel_clicked_cb),
-			  data);
-	g_signal_connect (G_OBJECT (btn_help), 
-			  "clicked",
-			  G_CALLBACK (help_cb),
-			  data);
 
-	g_signal_connect (G_OBJECT (data->add_key_button), 
-			  "clicked",
-			  G_CALLBACK (add_category_cb),
-			  data);
-	g_signal_connect (G_OBJECT (data->remove_key_button), 
-			  "clicked",
-			  G_CALLBACK (remove_category_cb),
-			  data);
+void
+dlg_categories_close  (GtkWidget *dlg)
+{
+	DialogData *data;
 
-	/* run dialog. */
+	g_return_if_fail (dlg != NULL);
 
-	{
-		GtkWidget *parent_win = window->viewer;
-		while (parent_win != NULL && !GTK_IS_WINDOW(parent_win))
-			parent_win = parent_win->parent;
-		if (parent_win != NULL)
-			gtk_window_set_transient_for (GTK_WINDOW (data->dialog), GTK_WINDOW (parent_win));
-	}
+	data = g_object_get_data (G_OBJECT (dlg), "dialog_data");
+	g_return_if_fail (data != NULL);
 
-	gtk_window_set_modal (GTK_WINDOW (data->dialog), FALSE);
-	gtk_widget_show (data->dialog);
-
-	gtk_widget_grab_focus (data->keywords_list_view);
+	gtk_widget_destroy (dlg);
 }
