@@ -28,21 +28,20 @@
 #include <string.h>
 #include <time.h>
 
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <libgnome/libgnome.h>
-#include <libgnomeui/gnome-dialog.h>
-#include <libgnomeui/gnome-dialog-util.h>
-#include <libgnomeui/gnome-file-entry.h>
 #include <libgnomeui/gnome-icon-theme.h>
 #include <libgnomeui/gnome-icon-lookup.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 #include <gphoto2/gphoto2-context.h>
 #include <gphoto2/gphoto2-camera.h>
 #include <gphoto2/gphoto2-abilities-list.h>
 #include <glade/glade.h>
 
 #include "gtk-utils.h"
-#include "gthumb-window.h"
+#include "gth-window.h"
+#include "gth-browser.h"
 #include "file-data.h"
 #include "file-utils.h"
 #include "dlg-categories.h"
@@ -73,7 +72,7 @@ typedef enum {
 
 
 typedef struct {
-	GThumbWindow   *window;
+	GthBrowser     *browser;
 	GladeXML       *gui;
 
 	GtkWidget      *dialog;
@@ -81,8 +80,7 @@ typedef struct {
 	GtkWidget      *import_preview_scrolledwindow;
 	GtkWidget      *camera_model_label; 
 	GtkWidget      *select_model_button;
-	GtkWidget      *destination_fileentry;
-	GtkWidget      *destination_entry;
+	GtkWidget      *destination_filechooserbutton;
 	GtkWidget      *film_entry;
 	GtkWidget      *keep_names_checkbutton;
 	GtkWidget      *delete_checkbutton;
@@ -148,8 +146,8 @@ static void
 destroy_cb (GtkWidget  *widget, 
 	    DialogData *data)
 {
-	GThumbWindow *window = data->window;
-	gboolean      thread_done;
+	GthBrowser *browser = data->browser;
+	gboolean    thread_done;
 
 	/* Remove check. */
 
@@ -177,7 +175,7 @@ destroy_cb (GtkWidget  *widget,
 	/**/
 
 	if (data->view_folder)
-		window_go_to_directory (data->window, data->local_folder);
+		gth_browser_go_to_directory (data->browser, data->local_folder);
 
 	/**/
 
@@ -203,7 +201,7 @@ destroy_cb (GtkWidget  *widget,
 
 	if (ImportPhotos) {
 		ImportPhotos = FALSE;
-		window_close (window);
+		gth_window_close (GTH_WINDOW (browser));
 	}
 }
 
@@ -681,7 +679,7 @@ get_mime_type_from_filename (const char*filename)
 {
 	const char *result;
 
-	char *n1 = g_filename_to_utf8 (filename, -1, 0, 0, 0);
+	char *n1 = g_filename_display_name (filename);
 	char *n2 = g_utf8_strdown (n1, -1);
 	char *n3 = g_filename_from_utf8 (n2, -1, 0, 0, 0);
 	result = gnome_vfs_mime_type_from_name_or_default (n3, NULL);
@@ -731,6 +729,7 @@ load_images_preview__step (AsyncOperationData *aodata,
 			    data->context);
 	
 	tmp_filename = get_temp_filename ();
+
 	if (gp_file_save (file, tmp_filename) >= 0) {
 		GdkPixbuf *pixbuf;
 		int        width, height;
@@ -762,11 +761,12 @@ load_images_preview__step (AsyncOperationData *aodata,
 						 NULL,
 						 fdata);
 		g_object_unref (pixbuf);
-		unlink (tmp_filename);
+		file_unlink (tmp_filename);
 	}
+
 	g_free (tmp_filename);
 	g_free (camera_folder);
-
+	
 	gp_file_unref (file);
 }
 
@@ -1038,16 +1038,18 @@ is_valid_filename (const char *name)
 static char*
 get_folder_name (DialogData *data)
 {
-	char *entry_val;
+	char *esc_destination;
 	char *destination;
 	char *film_name;
 	char *path;
 
-	entry_val = _gtk_entry_get_filename_text (GTK_ENTRY (data->destination_entry));
-	destination = remove_ending_separator (entry_val);
-	g_free (entry_val);
+	esc_destination = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (data->destination_filechooserbutton));
+	destination = gnome_vfs_unescape_string (esc_destination, "");
+	g_free (esc_destination);
 
-	if (! dlg_check_folder (data->window, destination)) {
+	eel_gconf_set_path (PREF_PHOTO_IMPORT_DESTINATION, destination);
+
+	if (! dlg_check_folder (GTH_WINDOW (data->browser), destination)) {
 		g_free (destination);
 		return NULL;
 	}
@@ -1070,8 +1072,6 @@ get_folder_name (DialogData *data)
 
 	path = g_build_filename (destination, film_name, NULL);
 	g_free (film_name);
-
-	eel_gconf_set_path (PREF_PHOTO_IMPORT_DESTINATION, destination);
 	g_free (destination);
 
 	return path;
@@ -1157,7 +1157,7 @@ add_categories_to_image (DialogData *data,
 	if (data->categories_list == NULL)
 		return;
 
-	cdata = comments_load_comment (filename);
+	cdata = comments_load_comment (filename, FALSE);
 	if (cdata == NULL)
 		cdata = comment_data_new ();
 	
@@ -1206,9 +1206,9 @@ save_image (DialogData *data,
 		data->interrupted = TRUE;
 		g_mutex_unlock (data->yes_or_no);
 	}
-	
+
+	g_free (camera_folder);	
 	g_free (local_path);
-	g_free (camera_folder);
 	gp_file_unref (file);
 }
 
@@ -1218,7 +1218,7 @@ add_film_keyword (const char *folder)
 {
 	CommentData *cdata;
 
-	cdata = comments_load_comment (folder);
+	cdata = comments_load_comment (folder, FALSE);
 	if (cdata == NULL)
 		cdata = comment_data_new ();
 	comment_data_add_keyword (cdata, _("Film"));
@@ -1251,8 +1251,7 @@ static void
 delete_images__done (AsyncOperationData *aodata, 
 		     DialogData         *data)
 {
-	GThumbWindow *window = data->window;
-	gboolean      interrupted;
+	gboolean interrupted;
 
 	task_terminated (data);
 
@@ -1267,7 +1266,7 @@ delete_images__done (AsyncOperationData *aodata,
 
 	if (ImportPhotos) {
 		ImportPhotos = FALSE;
-		gtk_widget_show (window->app);
+		gtk_widget_show (GTK_WIDGET (data->browser));
 	}
 
 	gtk_widget_destroy (data->dialog);
@@ -1436,7 +1435,7 @@ ok_clicked_cb (GtkButton  *button,
 	if (! ensure_dir_exists (data->local_folder, 0755)) {
 		char *utf8_path;
 		char *msg;
-		utf8_path = g_filename_to_utf8 (data->local_folder, -1, NULL, NULL, NULL);
+		utf8_path = g_filename_display_name (data->local_folder);
 		msg = g_strdup_printf (_("Could not create the folder \"%s\": %s"),
 				       utf8_path,
 				       errno_to_string ());
@@ -1453,7 +1452,7 @@ ok_clicked_cb (GtkButton  *button,
 	if (! check_permissions (data->local_folder, R_OK | W_OK | X_OK)) {
 		char *utf8_path;
 		char *msg;
-		utf8_path = g_filename_to_utf8 (data->local_folder, -1, NULL, NULL, NULL);
+		utf8_path = g_filename_display_name (data->local_folder);
 		msg = g_strdup_printf (_("You don't have the right permissions to create images in the folder \"%s\""), utf8_path);
 
 		display_error_dialog (data, _("Could not import photos"), msg);
@@ -1580,7 +1579,7 @@ import_delete_cb (GtkButton  *button,
 		camera_filename = file_name_from_path (camera_path);
 		
 		gp_camera_file_delete (data->camera, camera_folder, camera_filename, data->context);
-		g_free (camera_folder);
+		/* FIXME */
 	}
 
 	path_list_free (delete_list);
@@ -1687,17 +1686,17 @@ start_operation (DialogData    *data,
 
 
 void
-dlg_photo_importer (GThumbWindow *window)
+dlg_photo_importer (GthBrowser *browser)
 {
 	DialogData   *data;
 	GtkWidget    *btn_ok, *btn_cancel;
 	GdkPixbuf    *mute_pixbuf;
 	char         *default_path;
+	char         *esc_uri;
 	char         *default_film_name;
-	GValue        value = {0, };
 
 	data = g_new0 (DialogData, 1);
-	data->window = window;
+	data->browser = browser;
 
 	data->gui = glade_xml_new (GTHUMB_GLADEDIR "/" GLADE_FILE , NULL, NULL);
         if (!data->gui) {
@@ -1737,8 +1736,7 @@ dlg_photo_importer (GThumbWindow *window)
 	data->import_preview_scrolledwindow = glade_xml_get_widget (data->gui, "import_preview_scrolledwindow");
 	data->camera_model_label = glade_xml_get_widget (data->gui, "camera_model_label");
 	data->select_model_button = glade_xml_get_widget (data->gui, "select_model_button");
-	data->destination_fileentry = glade_xml_get_widget (data->gui, "destination_fileentry");
-	data->destination_entry = glade_xml_get_widget (data->gui, "destination_entry");
+	data->destination_filechooserbutton = glade_xml_get_widget (data->gui, "destination_filechooserbutton");
 	data->film_entry = glade_xml_get_widget (data->gui, "film_entry");
 	data->keep_names_checkbutton = glade_xml_get_widget (data->gui, "keep_names_checkbutton");
 	data->delete_checkbutton = glade_xml_get_widget (data->gui, "delete_checkbutton");
@@ -1766,14 +1764,6 @@ dlg_photo_importer (GThumbWindow *window)
 
 	/* Set widgets data. */
 
-	/* Make use of the new filechooser */
-
-	g_value_init (&value, G_TYPE_BOOLEAN);
-	g_value_set_boolean (&value, TRUE);
-	g_object_set_property (G_OBJECT (data->destination_fileentry), "use_filechooser", &value);
-
-	/**/
-
 	data->camera_present_pixbuf = gdk_pixbuf_new_from_file (GTHUMB_GLADEDIR "/" CAMERA_FILE, NULL);
 	mute_pixbuf = gdk_pixbuf_new_from_file (GTHUMB_GLADEDIR "/" MUTE_FILE, NULL);
 
@@ -1800,8 +1790,9 @@ dlg_photo_importer (GThumbWindow *window)
 	default_path = eel_gconf_get_path (PREF_PHOTO_IMPORT_DESTINATION, NULL);
 	if ((default_path == NULL) || (*default_path == 0))
 		default_path = g_strdup (g_get_home_dir());
-	gnome_file_entry_set_default_path (GNOME_FILE_ENTRY (data->destination_fileentry), default_path);
-	_gtk_entry_set_filename_text (GTK_ENTRY (data->destination_entry), default_path);
+	esc_uri = gnome_vfs_escape_host_and_path_string (default_path);
+	gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (data->destination_filechooserbutton), esc_uri);
+	g_free (esc_uri);
 	g_free (default_path);
 
 	default_film_name = eel_gconf_get_path (PREF_PHOTO_IMPORT_FILM, "");
@@ -1844,7 +1835,7 @@ dlg_photo_importer (GThumbWindow *window)
 
 	/* run dialog. */
 
-	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), GTK_WINDOW (window->app));
+	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), GTK_WINDOW (browser));
 	gtk_window_set_modal (GTK_WINDOW (data->dialog), FALSE);
 	gtk_widget_show (data->dialog);
 
