@@ -3,7 +3,7 @@
 /*
  *  GThumb
  *
- *  Copyright (C) 2003 Free Software Foundation, Inc.
+ *  Copyright (C) 2003, 2004, 2005 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,10 +23,13 @@
 
 #include <config.h>
 #include <string.h>
+
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <libgnome/libgnome.h>
+#include <libgnomevfs/gnome-vfs-mime.h>
 #include <libgnomevfs/gnome-vfs-file-info.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
+
 #include "file-utils.h"
 #include "glib-utils.h"
 #include "gth-exif-utils.h"
@@ -105,6 +108,7 @@ gth_exif_data_viewer_construct (GthExifDataViewer *edv)
 {
 	GtkCellRenderer   *renderer;
 	GtkTreeViewColumn *column;
+	GValue             value = { 0, };
 
 	edv->priv->scrolled_win = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (edv->priv->scrolled_win), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
@@ -140,6 +144,12 @@ gth_exif_data_viewer_construct (GthExifDataViewer *edv)
 							   renderer,
 							   "text", VALUE_COLUMN,
 							   NULL);
+
+	g_value_init (&value, PANGO_TYPE_ELLIPSIZE_MODE);
+	g_value_set_enum (&value, PANGO_ELLIPSIZE_END);
+	g_object_set_property (G_OBJECT (renderer), "ellipsize", &value);
+	g_value_unset (&value);
+
 	gtk_tree_view_append_column (GTK_TREE_VIEW (edv->priv->image_exif_view),
 				     column);
 
@@ -150,7 +160,7 @@ gth_exif_data_viewer_construct (GthExifDataViewer *edv)
 GType
 gth_exif_data_viewer_get_type ()
 {
-        static guint type = 0;
+        static GType type = 0;
 
         if (! type) {
                 GTypeInfo type_info = {
@@ -230,6 +240,7 @@ static ExifTag usefull_tags[] = {
 	EXIF_TAG_COMPRESSION,
 	EXIF_TAG_DOCUMENT_NAME,
 	EXIF_TAG_IMAGE_DESCRIPTION,
+	EXIF_TAG_MAKER_NOTE,
 
 	0,
 
@@ -332,9 +343,9 @@ get_entry_from_tag (ExifData *edata,
 
 
 static void
-update_exif_data (GthExifDataViewer *edv)
+update_exif_data (GthExifDataViewer *edv,
+		  ExifData          *edata)
 {
-	ExifData     *edata;
 	unsigned int  i;
 	gboolean      date_added = FALSE;
 	gboolean      aperture_added = FALSE;
@@ -344,7 +355,10 @@ update_exif_data (GthExifDataViewer *edv)
 	if (edv->priv->path == NULL)
 		return;
 
-	edata = exif_data_new_from_file (edv->priv->path);
+	if (edata == NULL)
+		edata = exif_data_new_from_file (edv->priv->path);
+	else
+		exif_data_ref (edata);
 
 	if (edata == NULL) 
                 return;
@@ -352,6 +366,7 @@ update_exif_data (GthExifDataViewer *edv)
 	for (i = 0; i < G_N_ELEMENTS (usefull_tags); i++) {
 		ExifEntry   *e;
 		GtkTreeIter  iter;
+		const char  *value;
 		char        *utf8_name;
 		char        *utf8_value;
 		
@@ -370,13 +385,23 @@ update_exif_data (GthExifDataViewer *edv)
 		if (e == NULL)
 			continue;
 
-		utf8_name = g_locale_to_utf8 (exif_tag_get_name (e->tag), -1, 0, 0, 0);
+		value = exif_tag_get_name (e->tag);
+		if (value == NULL)
+			continue;
+
+		utf8_name = g_locale_to_utf8 (value, -1, 0, 0, 0);
 		if (tag_is_present (GTK_TREE_MODEL (edv->priv->image_exif_model), utf8_name)) {
 			g_free (utf8_name);
 			continue;
 		}
+		
+		value = get_exif_entry_value (e);
+		if (value == NULL) {
+			g_free (utf8_name);
+			continue;
+		}
 
-		utf8_value = g_locale_to_utf8 (get_exif_entry_value (e), -1, 0, 0, 0);
+		utf8_value = g_locale_to_utf8 (value, -1, 0, 0, 0);
 		if ((utf8_value == NULL) 
 		    || (*utf8_value == 0) 
 		    || _g_utf8_all_spaces (utf8_value)) {
@@ -442,10 +467,9 @@ update_file_info (GthExifDataViewer *edv)
 	if (edv->priv->viewer == NULL)
 		return;
 
-	utf8_name = g_filename_to_utf8 (file_name_from_path (edv->priv->path), -1, 
-					NULL, NULL, NULL);
+	utf8_name = g_filename_display_basename (edv->priv->path);
 
-	if (!image_viewer_is_void(IMAGE_VIEWER (edv->priv->viewer))) {
+	if (!image_viewer_is_void (IMAGE_VIEWER (edv->priv->viewer))) {
 		width = image_viewer_get_image_width (edv->priv->viewer);
 		height = image_viewer_get_image_height (edv->priv->viewer);
 	} else {
@@ -493,6 +517,13 @@ update_file_info (GthExifDataViewer *edv)
 			    POS_COLUMN, -1,
 			    -1);
 
+	gtk_list_store_append (edv->priv->image_exif_model, &iter);
+	gtk_list_store_set (edv->priv->image_exif_model, &iter,
+			    NAME_COLUMN, _("Type"),
+			    VALUE_COLUMN, gnome_vfs_mime_get_description (gnome_vfs_get_file_mime_type (edv->priv->path, NULL, FALSE)),
+			    POS_COLUMN, -1,
+			    -1);
+
 	/**/
 
 	g_free (utf8_time_txt);
@@ -526,9 +557,9 @@ set_path (GthExifDataViewer *edv,
 void
 gth_exif_data_viewer_update (GthExifDataViewer *edv,
 			     ImageViewer       *viewer,
-			     const char        *path)
+			     const char        *path,
+			     gpointer           exif_data)
 {
-
 	set_path (edv, path);
 	if (viewer != NULL)
 		edv->priv->viewer = viewer;
@@ -542,7 +573,7 @@ gth_exif_data_viewer_update (GthExifDataViewer *edv,
 		update_file_info (edv);
 
 #ifdef HAVE_LIBEXIF
-	update_exif_data (edv);
+	update_exif_data (edv, exif_data);
 #endif /* HAVE_LIBEXIF */
 }
 
