@@ -3,7 +3,7 @@
 /*
  *  GThumb
  *
- *  Copyright (C) 2001, 2003 Free Software Foundation, Inc.
+ *  Copyright (C) 2001, 2003, 2004, 2005 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,150 +25,79 @@
 #include <glade/glade.h>
 #include <libgnomeui/gnome-window-icon.h>
 #include <libgnomeui/gnome-ui-init.h>
-#include <libbonoboui.h>
 #include <libgnomevfs/gnome-vfs-init.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
+#include <libbonobo.h>
 
-#include "main.h"
 #include "auto-completion.h"
 #include "catalog.h"
-#include "dir-list.h"
-#include "file-utils.h"
-#include "fullscreen.h"
-#include "gconf-utils.h"
-#include "gthumb-init.h"
-#include "gthumb-window.h"
-#include "image-viewer.h"
-#include "gth-image-list.h"
-#include "preferences.h"
-#include "icons/pixbufs.h"
-#include "typedefs.h"
 #include "comments.h"
+#include "dir-list.h"
+#include "main.h"
+#include "file-utils.h"
+#include "gconf-utils.h"
+#include "gth-application.h"
+#include "gth-browser.h"
+#include "gth-browser-actions-callbacks.h"
+#include "gth-image-list.h"
+#include "gth-viewer.h"
+#include "gthumb-init.h"
+#include "image-viewer.h"
 #include "pixbuf-utils.h"
-#include "commands-impl.h"
+#include "preferences.h"
+#include "typedefs.h"
 
+#include "icons/pixbufs.h"
 
 #define ICON_NAME_DIRECTORY "gnome-fs-directory"
 
-
-GList          *window_list = NULL;
-FullScreen     *fullscreen;
-char          **file_urls, **dir_urls;
-int             n_file_urls, n_dir_urls;
-int             StartInFullscreen = FALSE;
-int             StartSlideshow = FALSE;
-int             ViewFirstImage = FALSE;
-int             HideSidebar = FALSE;
-gboolean        ExitAll = FALSE;
-char           *ImageToDisplay = NULL;
-gboolean        FirstStart = TRUE;
-gboolean        ImportPhotos = FALSE;
+GthMonitor  *monitor = NULL;
+char       **file_urls, **dir_urls;
+int          n_file_urls, n_dir_urls;
+int          StartInFullscreen = FALSE;
+int          StartSlideshow = FALSE;
+int          ViewFirstImage = FALSE;
+int          HideSidebar = FALSE;
+gboolean     ExitAll = FALSE;
+char        *ImageToDisplay = NULL;
+gboolean     FirstStart = TRUE;
+gboolean     ImportPhotos = FALSE;
 
 static gboolean        view_comline_catalog = FALSE;
 static gboolean        view_single_image = FALSE;
 static GdkPixbuf      *folder_pixbuf = NULL;
-static GThumbWindow   *first_window = NULL;
+static GtkWidget      *first_window = NULL;
 static GnomeIconTheme *icon_theme = NULL;
-
-
-static void     prepare_app         ();
-static void     initialize_data     (poptContext pctx);
-static void     release_data        ();
-
-static void     init_session        (const char *argv0);
-static gboolean session_is_restored (void);
-static gboolean load_session        (void);
-
+static BonoboObject   *gth_application = NULL;
 
 struct poptOption options[] = {
 	{ "fullscreen", '\0', POPT_ARG_NONE, &StartInFullscreen, 0,
 	  N_("Start in fullscreen mode"),
 	  0 },
-
 	{ "slideshow", '\0', POPT_ARG_NONE, &StartSlideshow, 0,
 	  N_("Automatically start a slideshow"),
 	  0 },
-
 	{ "import-photos", '\0', POPT_ARG_NONE, &ImportPhotos, 0,
 	  N_("Automatically import digital camera photos"),
 	  0 },
-
 	{ NULL, '\0', 0, NULL, 0 }
 };
 
 
-/* -- Main -- */
-
-
-static gboolean 
-check_whether_to_set_fullscreen (gpointer data) 
+static void
+theme_changed_cb (GnomeIconTheme *theme, 
+		  gpointer        data)
 {
-	if ((first_window != NULL) && StartInFullscreen) 
-		fullscreen_start (fullscreen, first_window);
-	StartInFullscreen = FALSE;
-	return FALSE;
-}
-
-
-static gboolean 
-check_whether_to_import_photos (gpointer data) 
-{
-	if ((first_window != NULL) && ImportPhotos) 
-		file_camera_import_command_impl (NULL, first_window, NULL);		
-	return FALSE;
-}
-
-
-int 
-main (int argc, char *argv[])
-{
-	GnomeProgram *program;
-	GValue        value = { 0 };
-	poptContext   pctx;
-
-	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
-	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-	textdomain (GETTEXT_PACKAGE);
-
-	program = gnome_program_init ("gthumb", VERSION,
-				      LIBGNOMEUI_MODULE, 
-				      argc, argv,
-				      GNOME_PARAM_POPT_TABLE, options,
-				      GNOME_PARAM_HUMAN_READABLE_NAME, _("gThumb"),
-				      GNOME_PARAM_APP_PREFIX, GTHUMB_PREFIX,
-				      GNOME_PARAM_APP_SYSCONFDIR, GTHUMB_SYSCONFDIR,
-				      GNOME_PARAM_APP_DATADIR, GTHUMB_DATADIR,
-				      GNOME_PARAM_APP_LIBDIR, GTHUMB_LIBDIR,
-				      NULL);
-
-	if (! g_thread_supported ()) {
-		g_thread_init (NULL);
-		gdk_threads_init ();
+	if (folder_pixbuf != NULL) {
+		g_object_unref (folder_pixbuf);
+		folder_pixbuf = NULL;
 	}
-
-	g_object_get_property (G_OBJECT (program),
-			       GNOME_PARAM_POPT_CONTEXT,
-			       g_value_init (&value, G_TYPE_POINTER));
-	pctx = g_value_get_pointer (&value);
-	glade_gnome_init ();
-	gthumb_init ();
-	initialize_data (pctx);
-	poptFreeContext (pctx);
-	prepare_app ();
-
-	g_idle_add (check_whether_to_set_fullscreen, NULL);
-	g_idle_add (check_whether_to_import_photos, NULL);
-
-	bonobo_main ();
-
-	release_data ();
-
-	return 0;
+	all_windows_notify_update_icon_theme ();
 }
 
 
 static void
-create_default_categories_if_needed ()
+create_default_categories_if_needed (void)
 {
 	Bookmarks *categories;
 	char      *default_categories[] = { N_("Holidays"), 
@@ -205,15 +134,196 @@ create_default_categories_if_needed ()
 }
 
 
-static void
-theme_changed_cb (GnomeIconTheme *theme, 
-		  gpointer        data)
+static void 
+convert_old_comment (char     *real_file, 
+		     char     *rc_file, 
+		     gpointer  data)
 {
-	if (folder_pixbuf != NULL) {
-		g_object_unref (folder_pixbuf);
-		folder_pixbuf = NULL;
+	char *comment_file;
+	char *comment_dir;
+
+	comment_file = comments_get_comment_filename (real_file, TRUE, TRUE);
+	comment_dir = remove_level_from_path (comment_file);
+	ensure_dir_exists (comment_dir, 0755);
+
+	file_copy (rc_file, comment_file);
+
+	g_free (comment_dir);
+	g_free (comment_file);
+}
+
+
+static void
+convert_to_new_comment_system (void)
+{
+	if (!eel_gconf_get_boolean (PREF_MIGRATE_COMMENT_SYSTEM, TRUE))
+		return;
+	g_print ("converting comment system...");
+	visit_rc_directory_sync (RC_COMMENTS_DIR,
+				 COMMENT_EXT,
+				 "",
+				 TRUE,
+				 convert_old_comment,
+				 NULL);
+	g_print ("done.");
+	eel_gconf_set_boolean (PREF_MIGRATE_COMMENT_SYSTEM, FALSE);
+}
+
+
+/* SM support */
+
+
+/* argv[0] from main(); used as the command to restart the program */
+static const char *program_argv0 = NULL;
+
+/* The master client we use for SM */
+static GnomeClient *master_client = NULL;
+
+
+static void
+save_session (GnomeClient *client)
+{
+	const char  *prefix;
+	GList       *scan;
+	int          i = 0;
+
+	prefix = gnome_client_get_config_prefix (client);
+	gnome_config_push_prefix (prefix);
+
+	for (scan = gth_window_get_window_list (); scan; scan = scan->next) {
+		GthWindow  *window = scan->data;
+		char       *uri = NULL;
+		const char *location;
+		char       *key;
+
+		if (GTH_IS_VIEWER (window)) {
+			location = gth_window_get_image_filename (window);
+			if (location == NULL)
+				continue;
+
+			uri = g_strconcat ("file://", location, NULL);
+
+		} else {
+			GthBrowser *browser = (GthBrowser*) window;
+
+			switch (gth_browser_get_sidebar_content (browser)) {
+			case GTH_SIDEBAR_DIR_LIST:
+				location = gth_browser_get_current_directory (browser);
+				if (location == NULL)
+					continue;
+				uri = g_strconcat ("file://", location, NULL);
+				break;
+			
+			case GTH_SIDEBAR_CATALOG_LIST:
+				location = gth_browser_get_current_catalog (browser);
+				if (location == NULL)
+					continue;
+				uri = g_strconcat ("file://", location, NULL);
+				break;
+				
+			default:
+				break;
+			}
+		}
+
+		if (uri == NULL)
+			continue;
+
+		key = g_strdup_printf ("Session/location%d", i);
+		gnome_config_set_string (key, uri);
+
+		g_free (uri);
+		g_free (key);
+
+		i++;
 	}
-	all_windows_notify_update_icon_theme ();
+
+	gnome_config_set_int ("Session/locations", i);
+
+	gnome_config_pop_prefix ();
+	gnome_config_sync ();
+}
+
+
+/* save_yourself handler for the master client */
+static gboolean
+client_save_yourself_cb (GnomeClient *client,
+			 gint phase,
+			 GnomeSaveStyle save_style,
+			 gboolean shutdown,
+			 GnomeInteractStyle interact_style,
+			 gboolean fast,
+			 gpointer data)
+{
+	const char *prefix;
+	char       *argv[4] = { NULL };
+
+	save_session (client);
+
+	prefix = gnome_client_get_config_prefix (client);
+
+	/* Tell the session manager how to discard this save */
+
+	argv[0] = "rm";
+	argv[1] = "-rf";
+	argv[2] = gnome_config_get_real_path (prefix);
+	argv[3] = NULL;
+	gnome_client_set_discard_command (client, 3, argv);
+
+	/* Tell the session manager how to clone or restart this instance */
+
+	argv[0] = (char *) program_argv0;
+	argv[1] = NULL; /* "--debug-session"; */
+	
+	gnome_client_set_clone_command (client, 1, argv);
+	gnome_client_set_restart_command (client, 1, argv);
+
+	return TRUE;
+}
+
+
+/* die handler for the master client */
+static void
+client_die_cb (GnomeClient *client, gpointer data)
+{
+	if (! client->save_yourself_emitted)
+		save_session (client);
+
+	gtk_main_quit ();
+}
+
+
+static void
+init_session (const char *argv0)
+{
+	if (master_client != NULL)
+		return;
+
+	program_argv0 = argv0;
+
+	master_client = gnome_master_client ();
+
+	g_signal_connect (master_client, "save_yourself",
+			  G_CALLBACK (client_save_yourself_cb),
+			  NULL);
+
+	g_signal_connect (master_client, "die",
+			  G_CALLBACK (client_die_cb),
+			  NULL);
+}
+
+
+static gboolean
+session_is_restored (void)
+{
+	gboolean restored;
+	
+	if (! master_client)
+		return FALSE;
+
+	restored = (gnome_client_get_flags (master_client) & GNOME_CLIENT_RESTORED) != 0;
+
+	return restored;
 }
 
 
@@ -225,18 +335,16 @@ initialize_data (poptContext pctx)
 	int          argc;
 	char        *current_dir;
 	char        *path;
-	char        *pixmap_file;
 	int          i;
 
+	convert_to_new_comment_system (); 
 	create_default_categories_if_needed ();
 
 	eel_gconf_monitor_add ("/apps/gthumb");
 
-	fullscreen = fullscreen_new ();
+	monitor = gth_monitor_new ();
 
-	pixmap_file = PIXMAPSDIR "gthumb.png";
-	if (g_file_test (pixmap_file, G_FILE_TEST_EXISTS))
-		gnome_window_icon_set_default_from_file (pixmap_file);
+	/* Icon theme */
 
 	icon_theme = gnome_icon_theme_new ();
 	gnome_icon_theme_set_allow_svg (icon_theme, TRUE);
@@ -245,6 +353,12 @@ initialize_data (poptContext pctx)
 			  G_CALLBACK (theme_changed_cb),
 			  NULL);
 
+	/* Default windows icon */
+
+	gtk_window_set_default_icon_name ("gthumb");
+
+	/**/
+
 	init_session ("gthumb");
 	if (session_is_restored ()) 
 		return;
@@ -345,94 +459,162 @@ initialize_data (poptContext pctx)
 
 /* Free application data. */
 static void 
-release_data ()
+release_data (void)
 {
+	if (gth_application != NULL)
+                bonobo_object_unref (gth_application);
+
 	if (folder_pixbuf != NULL)
 		g_object_unref (folder_pixbuf);
 
 	g_object_unref (icon_theme);
 
-	fullscreen_close (fullscreen);
+	if (monitor != NULL) {
+		g_object_unref (monitor);
+		monitor = NULL;
+	}
+
 	preferences_release ();
 	eel_global_client_free ();
 	auto_compl_reset ();
 }
 
 
-/* Create the windows. */
 static void 
-prepare_app ()
+open_viewer_window (const char               *uri,
+		    gboolean                  use_factory,
+		    GNOME_GThumb_Application  app,
+		    CORBA_Environment        *env)
 {
-	GThumbWindow *current_window = NULL;
-	int           i;
-	
-	if (session_is_restored ()) {
-		load_session ();
+	GtkWidget *current_window;
+
+	if (use_factory) {
+		if (uri == NULL)
+			uri = "";
+		GNOME_GThumb_Application_open_viewer (app, uri, env);
 		return;
 	}
 
-	if (! view_comline_catalog 
-	    && (n_dir_urls == 0) 
-	    && (n_file_urls == 0)) {
+	current_window = gth_viewer_new (uri);
+	gtk_widget_show (current_window);
+	if (first_window == NULL)
+		first_window = current_window;
+}
+
+
+static void 
+open_browser_window (const char               *uri,
+		     gboolean                  show_window,
+		     gboolean                  use_factory,
+		     GNOME_GThumb_Application  app,
+		     CORBA_Environment        *env)
+{
+	GtkWidget *current_window;
+
+	if (use_factory) {
+		if (uri == NULL)
+			uri = "";
+		GNOME_GThumb_Application_open_browser (app, uri, env);
+		return;
+	}
+
+	current_window = gth_browser_new (uri);
+	if (show_window)
+		gtk_widget_show (current_window);
+	if (first_window == NULL)
+		first_window = current_window;
+}
+
+
+static void
+load_session (gboolean                  use_factory,
+	      GNOME_GThumb_Application  app,
+	      CORBA_Environment        *env)
+{
+	int i, n;
+	
+	gnome_config_push_prefix (gnome_client_get_config_prefix (master_client));
+
+	n = gnome_config_get_int ("Session/locations");
+	for (i = 0; i < n; i++) {
+		char *key;
+		char *location;
+
+		key = g_strdup_printf ("Session/location%d", i);
+		location = gnome_config_get_string (key);
+
+		if (pref_util_location_is_file (location) && path_is_file (location))
+			open_viewer_window (location, use_factory, app, env);
+		else
+			open_browser_window (location, TRUE, use_factory, app, env);
+
+		g_free (location);
+		g_free (key);
+	}
+
+	gnome_config_pop_prefix ();
+}
+
+
+/* Create the windows. */
+static void 
+prepare_app (void)
+{
+	CORBA_Object              factory;
+	gboolean                  use_factory = FALSE;
+	CORBA_Environment         env;
+	GNOME_GThumb_Application  app;
+	int                       i;
+	
+	factory = bonobo_activation_activate_from_id ("OAFIID:GNOME_GThumb_Application_Factory",
+                                                      Bonobo_ACTIVATION_FLAG_EXISTING_ONLY,
+                                                      NULL, NULL);
+
+	if (factory != NULL) {
+		use_factory = TRUE;
+		CORBA_exception_init (&env);
+		app = bonobo_activation_activate_from_id ("OAFIID:GNOME_GThumb_Application", 0, NULL, &env);
+	}
+
+	if (session_is_restored ()) {
+		load_session (use_factory, app, &env);
+		return;
+	}
+
+	if (! view_comline_catalog && (n_dir_urls == 0) && (n_file_urls == 0)) {
 		if (ImportPhotos)
 			preferences_set_startup_location (NULL);
-		current_window = window_new ();
-		if (!ImportPhotos)
-			gtk_widget_show (current_window->app);
-		if (first_window == NULL)
-			first_window = current_window;
+		open_browser_window (NULL, !ImportPhotos, use_factory, app, &env);
 	}
 
 	if (view_single_image) {
-		char *image_folder;
-
-		ImageToDisplay = g_strdup (file_urls[0]);
-		HideSidebar = TRUE;
-
-		image_folder = remove_level_from_path (ImageToDisplay);
-		preferences_set_startup_location (image_folder);
-		g_free (image_folder);
-
-		current_window = window_new ();
-		gtk_widget_show (current_window->app);
-
-		if (first_window == NULL)
-			first_window = current_window;
+		if (use_factory && eel_gconf_get_boolean (PREF_SINGLE_WINDOW, FALSE))
+			GNOME_GThumb_Application_load_image (app, file_urls[0], &env);
+		else
+			open_viewer_window (file_urls[0], use_factory, app, &env);
 
 	} else if (view_comline_catalog) {
 		char *catalog_uri;
 		char *catalog_path;
 		char *catalog_name;
 
-		catalog_name = g_strconcat (_("Command Line"), CATALOG_EXT, NULL);
-		catalog_path = get_catalog_full_path (catalog_name);
-		g_free (catalog_name);
-
-		catalog_uri = g_strconcat ("catalog://", catalog_path, NULL);
-		g_free (catalog_path);
-
-		preferences_set_startup_location (catalog_uri);
-		g_free (catalog_uri);
-
 		ViewFirstImage = TRUE;
 		HideSidebar = TRUE;
 
-		current_window = window_new ();
-		gtk_widget_show (current_window->app);
+		catalog_name = g_strconcat (_("Command Line"), CATALOG_EXT, NULL);
+		catalog_path = get_catalog_full_path (catalog_name);
+		catalog_uri = g_strconcat ("catalog://", catalog_path, NULL);
 
-		if (first_window == NULL)
-			first_window = current_window;
+		open_browser_window (catalog_uri, TRUE, use_factory, app, &env);
+
+		g_free (catalog_name);
+		g_free (catalog_path);
+		g_free (catalog_uri);
 	}
 
 	for (i = 0; i < n_dir_urls; i++) {
 		/* Go to the specified directory. */
-		preferences_set_startup_location (dir_urls[i]);
-
-		current_window = window_new ();
-		gtk_widget_show (current_window->app);
-
-		if (first_window == NULL)
-			first_window = current_window;
+		open_browser_window (dir_urls[i], TRUE, use_factory, app, &env);
 	}
 
 	/* Free urls. */
@@ -448,130 +630,135 @@ prepare_app ()
 			g_free (dir_urls[i]);
 		g_free (dir_urls);
 	}
+
+	/**/
+
+	if (use_factory) {
+		bonobo_object_release_unref (app, &env);
+		CORBA_exception_free (&env);
+		gdk_notify_startup_complete ();
+		exit (0);
+	} else
+		gth_application = gth_application_new (gdk_screen_get_default ());
 }
 
 
-void 
-all_windows_update_file_list ()
+static gboolean 
+check_whether_to_import_photos (gpointer data) 
 {
-	g_list_foreach (window_list, (GFunc) window_update_file_list, NULL);
+	if (ImportPhotos 
+	    && (first_window != NULL) 
+	    && GTH_IS_BROWSER (first_window))
+		gth_browser_activate_action_file_camera_import (NULL, GTH_BROWSER (first_window));
+
+	return FALSE;
 }
 
 
-void 
-all_windows_update_catalog_list ()
+int 
+main (int   argc, 
+      char *argv[])
 {
-	g_list_foreach (window_list, (GFunc) window_update_catalog_list, NULL);
-}
+	GnomeProgram *program;
+	GValue        value = { 0 };
+	poptContext   pctx;
 
+	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+	textdomain (GETTEXT_PACKAGE);
 
-void 
-all_windows_update_bookmark_list ()
-{
-	g_list_foreach (window_list, (GFunc) window_update_bookmark_list, NULL);
-}
+	program = gnome_program_init ("gthumb", VERSION,
+				      LIBGNOMEUI_MODULE, 
+				      argc, argv,
+				      GNOME_PARAM_POPT_TABLE, options,
+				      GNOME_PARAM_HUMAN_READABLE_NAME, _("gThumb"),
+				      GNOME_PARAM_APP_PREFIX, GTHUMB_PREFIX,
+				      GNOME_PARAM_APP_SYSCONFDIR, GTHUMB_SYSCONFDIR,
+				      GNOME_PARAM_APP_DATADIR, GTHUMB_DATADIR,
+				      GNOME_PARAM_APP_LIBDIR, GTHUMB_LIBDIR,
+				      NULL);
 
-
-void 
-all_windows_update_browser_options ()
-{
-	GList *scan;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow  *window = scan->data;
-		
-		window->file_list->enable_thumbs = eel_gconf_get_boolean (PREF_SHOW_THUMBNAILS, TRUE);
-		gth_file_list_set_thumbs_size (window->file_list, eel_gconf_get_integer (PREF_THUMBNAIL_SIZE, 95));
-		gth_file_view_set_view_mode (window->file_list->view,
-					     pref_get_view_mode ());
-		window_update_file_list (window);
-		dir_list_update_underline (window->dir_list);
+	if (! g_thread_supported ()) {
+		g_thread_init (NULL);
+		gdk_threads_init ();
 	}
+
+	g_object_get_property (G_OBJECT (program),
+			       GNOME_PARAM_POPT_CONTEXT,
+			       g_value_init (&value, G_TYPE_POINTER));
+
+	pctx = g_value_get_pointer (&value);
+	glade_gnome_init ();
+	gthumb_init ();
+	initialize_data (pctx);
+	poptFreeContext (pctx);
+	prepare_app ();
+	g_idle_add (check_whether_to_import_photos, NULL);
+
+	gtk_main ();
+
+	release_data ();
+	return 0;
+}
+
+
+/**/
+
+
+void 
+all_windows_update_catalog_list (void)
+{
+	gth_monitor_notify_reload_catalogs (monitor);
+}
+
+
+void 
+all_windows_notify_update_bookmarks (void)
+{
+	gth_monitor_notify_update_bookmarks (monitor);
+}
+
+
+void
+all_windows_notify_cat_files_created (const char *catalog_path,
+				      GList      *list)
+{
+	gth_monitor_notify_update_cat_files (monitor, 
+					     catalog_path, 
+					     GTH_MONITOR_EVENT_CREATED, 
+					     list);
+}
+
+
+void
+all_windows_notify_cat_files_deleted (const char *catalog_path,
+				      GList      *list)
+{
+	gth_monitor_notify_update_cat_files (monitor, 
+					     catalog_path, 
+					     GTH_MONITOR_EVENT_DELETED, 
+					     list);
 }
 
 
 void 
 all_windows_notify_files_created (GList *list)
 {
-	GList *scan;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-
-		if ((window->sidebar_content == GTH_SIDEBAR_DIR_LIST) &&
-		    window->monitor_enabled)
-			continue;
-
-		window_notify_files_created (window, list);
-	}
+	gth_monitor_notify_update_files (monitor, GTH_MONITOR_EVENT_CREATED, list);
 }
 
 
 void 
 all_windows_notify_files_deleted (GList *list)
 {
-	GList *scan;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-
-		/* FIXME
-		if ((window->sidebar_content == GTH_SIDEBAR_DIR_LIST) &&
-		    window->monitor_enabled)
-			continue;
-		*/
-
-		window_notify_files_deleted (window, list);
-	}
+	gth_monitor_notify_update_files (monitor, GTH_MONITOR_EVENT_DELETED, list);
 }
 
 
 void 
 all_windows_notify_files_changed (GList *list)
 {
-	GList *scan;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-
-		if ((window->sidebar_content == GTH_SIDEBAR_DIR_LIST) &&
-		    window->monitor_enabled)
-			continue;
-
-		window_notify_files_changed (window, list);
-	}
-}
-
-
-void 
-all_windows_notify_cat_files_added (const char *catalog_path,
-				    GList      *list)
-{
-	GList *scan;
-
-	if (list == NULL)
-		return;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-		window_notify_cat_files_added (window, catalog_path, list);
-	}
-}
-
-
-void 
-all_windows_notify_cat_files_deleted (const char *catalog_path,
-				      GList      *list)
-{
-	GList *scan;
-
-	if (list == NULL)
-		return;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-		window_notify_cat_files_deleted (window, catalog_path, list);
-	}
+	gth_monitor_notify_update_files (monitor, GTH_MONITOR_EVENT_CHANGED, list);
 }
 
 
@@ -579,23 +766,13 @@ void
 all_windows_notify_file_rename (const gchar *old_name,
 				const gchar *new_name)
 {
-	GList *scan;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-
-		if ((window->sidebar_content == GTH_SIDEBAR_DIR_LIST) &&
-		    window->monitor_enabled)
-			continue;
-
-		window_notify_file_rename (window, old_name, new_name);
-	}
+	gth_monitor_notify_file_renamed (monitor, old_name, new_name);
 }
 
 
 void
-all_windows_notify_files_rename (GList       *old_names,
-				 GList       *new_names)
+all_windows_notify_files_rename (GList *old_names,
+				 GList *new_names)
 {
 	GList *o_scan, *n_scan;
 
@@ -603,6 +780,7 @@ all_windows_notify_files_rename (GList       *old_names,
 		const char *old_name = o_scan->data;
 		const char *new_name = n_scan->data;
 
+		gth_monitor_notify_file_renamed (monitor, old_name, new_name);
 		all_windows_notify_file_rename (old_name, new_name);
 		
 		o_scan = o_scan->next;
@@ -615,138 +793,71 @@ void
 all_windows_notify_directory_rename (const gchar *old_name,
 				     const gchar *new_name)
 {
-	GList *scan;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-		window_notify_directory_rename (window, old_name, new_name);
-	}
+	gth_monitor_notify_directory_renamed (monitor, old_name, new_name);
 }
 
 
 void 
-all_windows_notify_directory_delete (const gchar *path)
+all_windows_notify_directory_delete (const char *path)
 {
-	GList *scan;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-		window_notify_directory_delete (window, path);
-	}
+	gth_monitor_notify_update_directory (monitor, path, GTH_MONITOR_EVENT_DELETED);
 }
 
 
 void 
-all_windows_notify_directory_new (const gchar *path)
+all_windows_notify_directory_new (const char *path)
 {
-	GList *scan;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-		window_notify_directory_new (window, path);
-	}
+	gth_monitor_notify_update_directory (monitor, path, GTH_MONITOR_EVENT_CREATED);
 }
 
 
 void
-all_windows_notify_catalog_rename (const gchar *oldname,
-				   const gchar *newname)
+all_windows_notify_catalog_rename (const char *old_name,
+				   const char *new_name)
 {
-	GList *scan;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-		window_notify_catalog_rename (window, oldname, newname);
-	}
+	gth_monitor_notify_catalog_renamed (monitor, old_name, new_name);
 }
 
 
 void
 all_windows_notify_catalog_new (const gchar *path)
 {
-	GList *scan;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-		window_notify_catalog_new (window, path);
-	}
+	gth_monitor_notify_update_catalog (monitor, path, GTH_MONITOR_EVENT_CREATED);
 }
 
 
 void
 all_windows_notify_catalog_delete (const gchar *path)
 {
-	GList *scan;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-		window_notify_catalog_delete (window, path);
-	}
+	gth_monitor_notify_update_catalog (monitor, path, GTH_MONITOR_EVENT_DELETED);
 }
 
 
 void 
-all_windows_notify_update_comment (const gchar *filename)
+all_windows_notify_update_metadata (const gchar *filename)
 {
-	GList *scan;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-		window_notify_update_comment (window, filename);
-	}
-}
-
-
-void 
-all_windows_notify_update_directory (const gchar *dir_path)
-{
-	GList *scan;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-
-		if ((window->sidebar_content == GTH_SIDEBAR_DIR_LIST) &&
-		    window->monitor_enabled)
-			continue;
-
-		window_notify_update_directory (window, dir_path);
-	}
+	gth_monitor_notify_update_metadata (monitor, filename);
 }
 
 
 void
-all_windows_notify_update_icon_theme ()
+all_windows_notify_update_icon_theme (void)
 {
-	GList *scan;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-		window_notify_update_icon_theme (window);
-	}
+	gth_monitor_notify_update_icon_theme (monitor);
 }
 
 
 void
-all_windows_remove_monitor ()
+all_windows_remove_monitor (void)
 {
-	GList *scan;
-	
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-		window_remove_monitor (window);
-	}
+	gth_monitor_pause (monitor);
 }
 
 
 void
-all_windows_add_monitor ()
+all_windows_add_monitor (void)
 {
-	GList *scan;
-	
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-		window_add_monitor (window);
-	}
+	gth_monitor_resume (monitor);
 }
 
 
@@ -806,176 +917,6 @@ get_folder_pixbuf (double icon_size)
 }
 
 
-
-
-/* SM support */
-
-/* The master client we use for SM */
-static GnomeClient *master_client = NULL;
-
-/* argv[0] from main(); used as the command to restart the program */
-static const char *program_argv0 = NULL;
-
-
-static void
-save_session (GnomeClient *client)
-{
-	const char  *prefix;
-	GList        *scan;
-	int          i = 0;
-
-	prefix = gnome_client_get_config_prefix (client);
-	gnome_config_push_prefix (prefix);
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-		char         *location = NULL;
-		char         *key;
-
-		if ((window->sidebar_content == GTH_SIDEBAR_DIR_LIST) 
-		    && (window->dir_list != NULL)
-		    && (window->dir_list->path != NULL))
-			location = g_strconcat ("file://",
-						window->dir_list->path,
-						NULL);
-		else if ((window->sidebar_content == GTH_SIDEBAR_CATALOG_LIST) 
-			 && (window->catalog_path != NULL))
-			location = g_strconcat ("catalog://",
-						window->catalog_path,
-						NULL);
-		
-		if (location == NULL)
-			continue;
-
-		key = g_strdup_printf ("Session/location%d", i);
-		gnome_config_set_string (key, location);
-
-		g_free (key);
-		g_free (location);
-
-		i++;
-	}
-
-	gnome_config_set_int ("Session/locations", i);
-
-	gnome_config_pop_prefix ();
-	gnome_config_sync ();
-}
-
-
-/* save_yourself handler for the master client */
-static gboolean
-client_save_yourself_cb (GnomeClient *client,
-			 gint phase,
-			 GnomeSaveStyle save_style,
-			 gboolean shutdown,
-			 GnomeInteractStyle interact_style,
-			 gboolean fast,
-			 gpointer data)
-{
-	const char *prefix;
-	char       *argv[4] = { NULL };
-
-	save_session (client);
-
-	prefix = gnome_client_get_config_prefix (client);
-
-	/* Tell the session manager how to discard this save */
-
-	argv[0] = "rm";
-	argv[1] = "-rf";
-	argv[2] = gnome_config_get_real_path (prefix);
-	argv[3] = NULL;
-	gnome_client_set_discard_command (client, 3, argv);
-
-	/* Tell the session manager how to clone or restart this instance */
-
-	argv[0] = (char *) program_argv0;
-	argv[1] = NULL; /* "--debug-session"; */
-	
-	gnome_client_set_clone_command (client, 1, argv);
-	gnome_client_set_restart_command (client, 1, argv);
-
-	return TRUE;
-}
-
-
-/* die handler for the master client */
-static void
-client_die_cb (GnomeClient *client, gpointer data)
-{
-	if (! client->save_yourself_emitted)
-		save_session (client);
-
-	gtk_main_quit ();
-}
-
-
-static void
-init_session (const char *argv0)
-{
-	if (master_client != NULL)
-		return;
-
-	program_argv0 = argv0;
-
-	master_client = gnome_master_client ();
-
-	g_signal_connect (master_client, "save_yourself",
-			  G_CALLBACK (client_save_yourself_cb),
-			  NULL);
-
-	g_signal_connect (master_client, "die",
-			  G_CALLBACK (client_die_cb),
-			  NULL);
-}
-
-
-static gboolean
-session_is_restored (void)
-{
-	gboolean restored;
-	
-	if (! master_client)
-		return FALSE;
-
-	restored = (gnome_client_get_flags (master_client) & GNOME_CLIENT_RESTORED) != 0;
-
-	return restored;
-}
-
-
-static gboolean
-load_session (void)
-{
-	int i, n;
-	
-	gnome_config_push_prefix (gnome_client_get_config_prefix (master_client));
-
-	n = gnome_config_get_int ("Session/locations");
-	for (i = 0; i < n; i++) {
-		GThumbWindow *window;
-		char         *key;
-		char         *location;
-
-		key = g_strdup_printf ("Session/location%d", i);
-		location = gnome_config_get_string (key);
-		g_free (key);
-
-		preferences_set_startup_location (location);
-
-		window = window_new ();
-		gtk_widget_show (window->app);
-
-		g_free (location);
-	}
-
-	gnome_config_pop_prefix ();
-
-	return TRUE;
-}
-
-
 int
 get_folder_pixbuf_size_for_list (GtkWidget *widget)
 {
@@ -1008,7 +949,7 @@ folder_is_film (const char *folder)
 
 	folder = pref_util_remove_prefix (folder);
 
-	cdata = comments_load_comment (folder);
+	cdata = comments_load_comment (folder, FALSE);
 	if (cdata != NULL) {
 		int i;
 		for (i = 0; i < cdata->keywords_n; i++)
